SUBROUTINE FESlocal1
! FES estimator for ncv=1 [J. Phys. Chem. B 2015, 119, 736-742]
use ftsmod

implicit none
! local variables
integer igr1, igs, ng1
real delt1, cofg1(ngrid(1)), gpoints(ngrid(1)), expval 
real pmf0, pmf(ngrid(1)), sumg
character :: wrtline*2048

! ** Grid spacing, coefficients and points in CV space:
ng1 = ngrid(1)
delt1 = (cvlim(2,1)-cvlim(1,1))/(ng1-1.0)
cofg1 = 1.0
cofg1(1) = 0.5
cofg1(ng1) = 0.5
do igr1 = 1, ng1
  gpoints(igr1) = cvlim(1,1) + (igr1-1.0)*delt1
enddo

! ** Computation of FES:
write(wrtline,*) '# FES estimator:'
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# Number of CVs = 1'
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# System limits = [',cvlim(1,1),',',cvlim(2,1),']'
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# Number of grid points = ',ng1
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# state --- FES --- FES (local estimator)'
write(ufel,'(a)') trim(adjustl(wrtline))
pmf0 = 0.0
pmf = 0.0
do igs = 1, ngauss
  sumg = 0.0
  ! FES at selected CV point
  expval = -(cvpoint(1)-zgauss(1,igs))**2*dgauss(1)*0.5
  pmf0 = pmf0 - ctgauss*wgauss(igs)*exp(expval)
  ! FES at the grid points
  do igr1 = 1, ng1
    expval = -(gpoints(igr1)-zgauss(1,igs))**2*dgauss(1)*0.5
    pmf(igr1) = pmf(igr1) - ctgauss*wgauss(igs)*exp(expval)
    sumg = sumg + exp(-pmf(igr1)*ikbt)*cofg1(igr1)
  enddo  
  write(wrtline,*) igs,pmf0,pmf0+kbt*log(delt1*sumg)
  write(ufel,'(a)') trim(adjustl(wrtline))
enddo

return
END SUBROUTINE FESlocal1
