SUBROUTINE FESlocal3
! FES estimator for ncv=3 [J. Phys. Chem. B 2015, 119, 736-742]
use ftsmod

implicit none
! local variables
integer igr1, igr2, igr3, igs, ngd1, ngd2, ng3, ip, ip0, ip1, icv
real delt1, delt2, delt3, cofg1(ngrid(1)), cofg2(ngrid(2)), cofg3(ngrid(3)), gpoints(ngrid(1)*ngrid(2)*ngrid(3),3), expval 
real pmf0, pmf(ngrid(1)*ngrid(2)*ngrid(3)), sumg
character :: wrtline*2048

! ** Grid spacing, coefficients and points in CV space:
ngd1 = ngrid(1)
ngd2 = ngrid(2)
ng3 = ngrid(3)
delt1 = (cvlim(2,1)-cvlim(1,1))/(ngd1-1.0)
delt2 = (cvlim(2,2)-cvlim(1,2))/(ngd2-1.0)
delt3 = (cvlim(2,3)-cvlim(1,3))/(ng3-1.0)
cofg1 = 1.0
cofg1(1) = 0.5
cofg1(ngd1) = 0.5
cofg2 = 1.0
cofg2(1) = 0.5
cofg2(ngd2) = 0.5
cofg3 = 1.0
cofg3(1) = 0.5
cofg3(ng3) = 0.5
do igr1 = 1, ngd1
  ip0 = (igr1-1)*ngd2*ng3
  do igr2 = 1, ngd2
    ip1 = ip0 + (igr2-1)*ng3
    do igr3 = 1, ng3
      ip = ip1 + igr3
      gpoints(ip,1) = cvlim(1,1) + (igr1-1.0)*delt1
      gpoints(ip,2) = cvlim(1,2) + (igr2-1.0)*delt2
      gpoints(ip,3) = cvlim(1,3) + (igr3-1.0)*delt3
    enddo
  enddo
enddo

! ** Computation of FES:
write(wrtline,*) '# FES estimator:'
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# Number of CVs = 3'
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# System limits = [',cvlim(1,1),',',cvlim(2,1),'];[',cvlim(1,2),',',cvlim(2,2),'];[',cvlim(1,3),',',cvlim(2,3),']'
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# Number of grid points = ',ngd1*ngd2*ng3
write(ufel,'(a)') trim(adjustl(wrtline))
write(wrtline,*) '# state --- FES --- FES (local estimator)'
write(ufel,'(a)') trim(adjustl(wrtline))
pmf0 = 0.0
pmf = 0.0
do igs = 1, ngauss
  sumg = 0.0
  ! FES at selected CV point
  expval = 0.0
  do icv = 1, 3
    expval = expval - (cvpoint(icv)-zgauss(icv,igs))**2*dgauss(icv)*0.5
  enddo
  pmf0 = pmf0 - ctgauss*wgauss(igs)*exp(expval)
  ! FES at the grid points
  do igr1 = 1, ngd1
    ip0 = (igr1-1)*ngd2*ng3
    do igr2 = 1, ngd2
      ip1 = ip0 + (igr2-1)*ng3
      do igr3 = 1, ng3
        ip = ip1 + igr3
        expval = 0.0
        do icv = 1, 3
          expval = expval - (gpoints(ip,icv)-zgauss(icv,igs))**2*dgauss(icv)*0.5
        enddo
        pmf(ip) = pmf(ip) - ctgauss*wgauss(igs)*exp(expval)
        sumg = sumg + exp(-pmf(ip)*ikbt)*cofg1(igr1)*cofg2(igr2)*cofg3(igr3)
      enddo
    enddo 
  enddo  
  write(wrtline,*) igs,pmf0,pmf0+kbt*log(delt1*delt2*delt3*sumg)
  write(ufel,'(a)') trim(adjustl(wrtline))
enddo

return
END SUBROUTINE FESlocal3
