double precision function MRG32k3a()
! L'ECUYER, P. "Good parameters and implementations for combined multiple recusrive 
!               random numbers generator", Operat. Res. 47, 1, 159-164. 1999
! period length = 2^{191}
! MRG32k3a output sequence is a good imitation of a sequence of independent uniform 
! random variables over the real interval (0,1)

use rngmod
implicit none 
! local variables
integer*8 :: k
double precision :: p1, p2

! Component 1
p1 = a12*s11 - a13*s10
k = p1/m1
p1 = p1 - k*m1
if (p1<0.0) p1 = p1 + m1
s10 = s11
s11 = s12
s12 = p1 

! Component 2
p2 = a21*s22 - a23*s20
k = p2/m2
p2 = p2 - k*m2
if (p2<0.0) p2 = p2 + m2
s20 = s21
s21 = s22
s22 = p2

! Combination
if (p1<=p2) then
  MRG32k3a = (p1-p2+m1)*norm
else
  MRG32k3a = (p1-p2)*norm
endif

end function
