SUBROUTINE bsplinec
! Obtention of the coefficients for the bicubic interpolation.
! First and cross derivatives are obtained from cubic spline interpolations
use ftsmod

implicit none
! local variables
integer j, k, l, nx, ny, ipx
real dx, dy
real ya(ng(1),ng(2)), y1a(ng(1),ng(2)), y2a(ng(1),ng(2)), y12a(ng(1),ng(2)), y2tmp(mxgrid), dsplint
real y(4), y1(4), y2(4), y12(4)

nx = ng(1)
ny = ng(2)
dx = dcel(1)
dy = dcel(2)
! ** First derivative
do j = 1, nx
  ipx = (j-1)*ny
  do k = 1, ny
    ya(j,k) = fesg(ipx+k)
  enddo
  call spline(ya(j,:),ny,dy,y2tmp(1:ny))
  do k = 1, ny
    y2a(j,k)= dsplint(xg(1:ny,2),ya(j,:),y2tmp(1:ny),ny,dy,xg(k,2))
  enddo
enddo
do k = 1, ny
  call spline(ya(:,k),nx,dx,y2tmp(1:nx))
  do j = 1, nx
    y1a(j,k) = dsplint(xg(1:nx,1),ya(:,k),y2tmp(1:nx),nx,dx,xg(j,1))
  enddo
enddo
! ** Cross derivative
do j = 1, nx
  call spline(y1a(j,:),ny,dy,y2tmp(1:ny))
  do k = 1, ny
    y12a(j,k) = dsplint(xg(1:ny,2),y1a(j,:),y2tmp(1:ny),ny,dy,xg(k,2))
  enddo
enddo
do k = 1, ny
  call spline(y2a(:,k),nx,dx,y2tmp(1:nx))
  do j = 1, nx
    y12a(j,k) = (y12a(j,k) + dsplint(xg(1:nx,1),y2a(:,k),y2tmp(1:nx),nx,dx,xg(j,1)))*0.5
  enddo
enddo
! ** Tabulated coefficients 
do k = 1, ny-1
  do j = 1, nx-1
    ! Function at the four gird points
    y(1) = ya(j,k)
    y(2) = ya(j+1,k)
    y(3) = ya(j+1,k+1)
    y(4) = ya(j,k+1)
    ! Gradients at the four grid points
    y1(1) = y1a(j,k)
    y1(2) = y1a(j+1,k)
    y1(3) = y1a(j+1,k+1)
    y1(4) = y1a(j,k+1)
    y2(1) = y2a(j,k)
    y2(2) = y2a(j+1,k)
    y2(3) = y2a(j+1,k+1)
    y2(4) = y2a(j,k+1)
    ! Cross derivative at the four grid points
    y12(1) = y12a(j,k)
    y12(2) = y12a(j+1,k)
    y12(3) = y12a(j+1,k+1)
    y12(4) = y12a(j,k+1)
    ! Coefficients for the bicubic interpolation
    l = (nx-1)*(k-1)+j
    call bcucof(y,y1,y2,y12,dx,dy,cg(:,:,l))
  enddo
enddo

END SUBROUTINE bsplinec
