real FUNCTION fesval2(x,y)
! B-spline or bicubic interpolation of FES at(x,y) point in the 2D CV
use ftsmod

implicit none
! INPUT VARIABLES
real x, y
! LOCAL VARIABLES
integer ix, iy, nxmax, nymax, nx, ny, ipx, ipy, ind
real xl, yl, vab1, vab2
real mx, my, m2
real t, u

fesval2 = 0.0
! CV coordinates in the local grid system
xl = x - xg(1,1)
yl = y - xg(1,2)
if (QBspline) then ! B-spline interpolation
  nxmax = ng(1)
  nymax = ng(2)
  ! integer counter for CV coordinates
  ix = nint(xl*idcel(1)) 
  iy = nint(yl*idcel(2))
  if (ix==0) ix = 1
  if (iy==0) iy = 1
  if (ix==(nxmax-1)) ix = nxmax - 2
  if (iy==(nymax-1)) iy = nymax - 2
  ! distribution by 9 adjacent grid points
  do nx = ix-1, ix+1
    ipx = nx*nymax
    vab1 = 1.5 + (nx*dcel(1)-xl)*idcel(1)
    vab2 = vab1 - 1.0
    mx = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
    do ny = iy-1, iy+1
      ipy = ny + ipx + 1
      vab1 = 1.5 + (ny*dcel(2)-yl)*idcel(2)
      vab2 = vab1 - 1.0
      my = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
      fesval2 = fesval2 + mx*my*fesg(ipy)
    enddo
  enddo
elseif (Qbicubic) then ! bicubic interpolation
 ! integer counter for CV coordinates and coefficients for the bicubic interpolation
 ix = int(xl*idcel(1)) + 1
 iy = int(yl*idcel(2)) + 1
 ind = (ng(1)-1)*(iy-1) + ix
 ! interpolated function value
 t = (x-xg(ix,1))*idcel(1)
 u = (y-xg(iy,2))*idcel(2)
 do nx = 4, 1, -1
   fesval2 = t*fesval2 + ((cg(nx,4,ind)*u+cg(nx,3,ind))*u + cg(nx,2,ind))*u + cg(nx,1,ind)
 enddo
endif


END FUNCTION fesval2
