SUBROUTINE fts
! Finite Temperature String (FTS) algorithm
! Note: mig > 3 (this is not tested in this routine)
use ftsmod
use errormod

implicit none
! local variables
integer iter, i, j, aval, afunc
real rhoinit(ncv,mig), zav(ncv,mig), imig, idrho
real fact, random(ncv), vec2(ncv), force(ncv), tmp(ncv), dist, distmx, distM
double precision vec(ncv)
real riter, drho2, rhot(ncv,mig), d(ncv,mig-2), rhold(ncv,mig)
real  deucl(mig), cte 
character :: wrtline*2048
logical*1 convergence, ok

fact = sqrt(2.0*kbt*dt)
if (Qestring) then
  do i = 1, mig
    do j = 1, ncv
      rhoinit(j,i) = rho(j,i)
    enddo
  enddo
  imig = 1.0/mig
endif
if (Qconv) idrho = 1.0/drho
do i = 1, mig
  do j = 1, ncv
    zav(j,i) = 0
  enddo
enddo
iter = 1
convergence = .false.
do while (.not.convergence .and. iter<=ftsit)
  ! ** Step1: computing running average of each replica and updating the replicas
  do i = 1, mig
    ! **** 1.a) computing running average of each replica
    do j = 1, ncv
      zav(j,i) = zav(j,i) + zrep(j,i)
    enddo
    ! **** 1.b) updating step via discretized-time overdamped Langevin eq. in CV space
    call rgauss2(vec) ! a vector of ncv pseudorandom number is generated, each from a standard normal distribution
    random = matmul(sigmat,vec) 
    call gradpmf(vec2,i) ! force vector associated to the FES
    force = matmul(Mm,vec2)
    do j = 1, ncv
       tmp(j) = zrep(j,i) + dt*force(j) + fact*random(j)
    enddo
    if (Qangcv) then
      do j = 1, ncv
        if (angcv(j) .and. abs(tmp(j)).gt.dcv(j)) tmp(j) = sign(dcv(j),tmp(j))
      enddo
    endif
    ! **** 1.c) acceptance/rejection step (reflecting boundary condition)
    dist = distM(tmp(1:ncv),rho(1:ncv,i)) 
    j = 0
    ok = .true.
    do while (j.lt.mig .and. ok)
      j = j + 1
      if (j.ne.i) ok = dist < distM(tmp(1:ncv),rho(1:ncv,j))
    enddo
    if (ok) then
      do j = 1, ncv
        zrep(j,i) = tmp(j)
      enddo
    endif
  enddo ! next image
  ! ** Step 2: updating the images and reparametrization
  if (Qconv) then
    do i = 1, mig
      do j = 1, ncv
        rhold(j,i) = rho(j,i)
      enddo
    enddo
  endif
  ! **** 2.a) updating the images
  riter = 1.0/iter
  drho2 = 1.0 - drho
  do j = 1, ncv
    rhot(j,1) = drho2*rho(j,1)+drho*zav(j,1)*riter
    rhot(j,mig) = drho2*rho(j,mig)+drho*zav(j,mig)*riter
    d(j,1) = drho2*rho(j,2) + drho*zav(j,2)*riter + kappt*rhot(j,1)
    d(j,mig-2) = drho2*rho(j,mig-1) + drho*zav(j,mig-1)*riter + kappt*rhot(j,mig)
    do i = 2, mig-3
      d(j,i) = drho2*rho(j,i+1) + drho*zav(j,i+1)*riter
    enddo
  enddo
  call solve_tridiag(d,rhot(1:ncv,2:mig-1))
  ! **** 2.b) reparametrization
  ! ****** 2.b.i) euclidean distances between consecutives images
  do i = 2, mig 
    deucl(i) = 0.0 
    do j = 1, ncv 
      deucl(i) = deucl(i) + (rhot(j,i)-rhot(j,i-1))**2 
    enddo 
    deucl(i) = sqrt(deucl(i)) 
  enddo 
  ! ****** 2.b.ii) length of the piecewise linear curve up to a given image (recurrence relation)
  Lcur(1) = 0.0 
  do i = 2, mig  
    Lcur(i) = Lcur(i-1) + deucl(i) 
    deucl(i) = 1.0/deucl(i) 
  enddo 
  ! ****** 2.b.iii) distance from image 1 up to the point at which we need to set a new given image (recurrence relation)
  cte = Lcur(mig)/(mig-1.0) 
  ldist(1) = 0.0 
  do i = 2, mig 
    ldist(i) = ldist(i-1) + cte 
  enddo 
  ! ****** 2.b.iv) new equidistant images
  do j = 1, ncv
    rho(j,1) = rhot(j,1)
    rho(j,mig) = rhot(j,mig) 
  enddo
  do i = 2, mig-1 
    aval = afunc(i) 
    do j = 1, ncv 
      rho(j,i) = rhot(j,aval-1) + (ldist(i)-Lcur(aval-1))*(rhot(j,aval)-rhot(j,aval-1))*deucl(aval) 
    enddo 
  enddo 
  ! ** Step 3: ressetting step
  do i = 1, mig
    dist = distM(zrep(1:ncv,i),rho(1:ncv,i))
    j = 0
    ok = .true.
    do while (j.lt.mig .and. ok)
      j = j + 1
      if (j.ne.i) ok = dist < distM(zrep(1:ncv,i),rho(1:ncv,j))
    enddo
    if (.not.ok) then
      do j = 1, ncv
        zrep(j,i) = rho(j,i)
      enddo
    endif
  enddo
  ! ** Euclidean distance of the string from the initial condition
  if (Qestring) then
    dist = 0.0
    do i = 1, mig
      do j = 1, ncv
        dist = dist + (rho(j,i)-rhoinit(j,i))**2
      enddo
    end do
    dist = sqrt(dist)*imig
    write(wrtline,*) iter,dist
    write(ustring,'(a)') trim(adjustl(wrtline))
  endif
  ! ** Check convergence
  if (Qconv) then
    ! **** maximum displacement of the string
    distmx = 0.0
    do i = 1, mig
      dist = 0.0
      do j = 1, ncv
        dist = dist + (rho(j,i)-rhold(j,i))**2
      enddo
      dist = sqrt(dist)*idrho
      if (distmx<dist) distmx = dist
    enddo
    convergence = distmx<ftstol
  endif
  iter = iter + 1
enddo
if (Qconv .and. .not.convergence) call error ('fts', 'Convergence is not achieved', faterr)

! Print images file
if (Qfts) then
  write(wrtline,*) '! Angle CVs?:'
  write(ufts,'(a)') trim(adjustl(wrtline))
  write(wrtline,*) Qangcv
  write(ufts,'(a)') trim(adjustl(wrtline))
  if (Qangcv) then
    write(wrtline,*) '! Constant module for angle CVs:'
    write(ufts,'(a)') trim(adjustl(wrtline))
    write(wrtline,*) (dcv(j),j=1,ncv)
    write(ufts,'(a)') trim(adjustl(wrtline))
  endif
  write(wrtline,*) '! Images (ncv dimension):' 
  write(ufts,'(a)') trim(adjustl(wrtline))
  do i = 1, mig
    write(wrtline,*) (rho(j,i),j=1,ncv)
    write(ufts,'(a)') trim(adjustl(wrtline))
  enddo
endif

return
END SUBROUTINE fts
