SUBROUTINE ftsfe
! Algorithm for estimating the free energy along a given path and the width of the transition path
use ftsmod

implicit none
! local variables
integer iter, i, j
real fact, random(ncv), vec2(ncv), force(ncv), tmp(ncv), dist, distM, pmf
double precision vec(ncv)
real imig
character :: wrtline*2048
logical*1 ok

! ** Step1: computing the width of the transition path (reaction tube)
if (Qtube) then
  do i = 1, mig
    do j = 1, ncv
      zrep(j,i) = rho(j,i)
    enddo
  enddo
  fact = sqrt(2.0*kbt*dt)
  do iter = 1, nfts  
    if (iter>nftskip) then
      do i = 1, mig
        write(wrtline,*) (zrep(j,i),j=1,ncv)
        write(utube,'(a)') trim(adjustl(wrtline))
      enddo
    endif
    do i = 1, mig 
      ! **** 1.a) updating step via discretized-time overdamped Langevin eq. in CV space
      call rgauss2(vec) ! a vector of ncv pseudorandom number is generated, each from a standard normal distribution
      random = matmul(sigmat,vec)
      call gradpmf(vec2,i) ! force vector associated to the FES
      force = matmul(Mm,vec2)
      do j = 1, ncv
        tmp(j) = zrep(j,i) + dt*force(j) + fact*random(j)
      enddo
      if (Qangcv) then
        do j = 1, ncv
          if (angcv(j) .and. abs(tmp(j)).gt.dcv(j)) tmp(j) = sign(dcv(j),tmp(j))
        enddo
      endif
      ! **** 1.b) acceptance/rejection step (reflecting boundary condition)
      dist = distM(tmp(1:ncv),rho(1:ncv,i))
      j = 0
      ok = .true.
      do while (j<mig .and. ok)
        j = j + 1
        if (j.ne.i) ok = dist < distM(tmp(1:ncv),rho(1:ncv,j))
      enddo
      if (ok) then
        do j = 1, ncv
          zrep(j,i) = tmp(j)
        enddo
      endif
    enddo ! next image
  enddo ! next iteration
endif
! ** Step2: free energy along the path
write(wrtline,*) '# reaction coordinate --- free energy [KJ/mole]'
write(ufe,'(a)') trim(adjustl(wrtline))
imig = 1.0/(mig-1.0)
do i = 1, mig
  write(wrtline,*) (i-1)*imig,pmf(i,1)
  write(ufe,'(a)') trim(adjustl(wrtline))
enddo

return
END SUBROUTINE ftsfe
