! Variables related to the string method (FTSM and STRM modules)
! ==============================================================
!
! ncv                -> Number of Collective Variables (CVs)
! mig                -> Number of discrete images/replicas 
! Mm(ncv,ncv)        -> Real and symmetric metric tensor
! Um(ncv,ncv)        -> Transformation matrix for the orthogonal transformation Um^t Mm Um
! lamb(ncv)          -> Eigenvalues of Mm
! Mminv(ncv,ncv)     -> Metric tensor (inverse)
! sigmat(ncv,ncv)    -> Random tensor
! Qftsm              -> Print matrices file
! uftsm              -> Unit for matrices file
! uftsm2             -> Unit for metric tensor file
! Qfts               -> Print images file
! ufts               -> Unit for images file
! ufts2              -> Unit for end images file
! zrep(ncv,mig)      -> Replicas
! rho(ncv,mig)       -> Images
! ngskip             -> Number of lines to skip in the hills file (input file)
! ngauss             -> Number of Gaussians functions 
! nigauss            -> Initial Gaussians for average FES
! ugauss             -> Unit for hills file (input file)
! zgauss(ncv,ngauss) -> Values of CVs for each Gaussian function
! dgauss(ncv)        -> Gaussian width for each CV
! wgauss(ngauss)     -> Gaussian height for each Gaussian function
! (kappt,drho)       -> Adjustable parameters for updating string images
! Lcur(mig)          -> Length of the piecewise linear curve up to a given image
! ldist(mig)         -> Distance from image 1 up to the point at which we need to set a new given image
! Qconv              -> Convergence criteria (Qconv=.false. => maximum number of iterations)
! ftsit              -> Maximum nuber of iterations 
! ftstol             -> Tolerance (Qconv=.true.)
! Qtube              -> Print width of the transition path (reaction tube)
! nfts               -> Number of time steps (reaction tube calculation)
! nftskip            -> Number of iterations after which reaction tube is printed
! utube              -> Unit for reaction tube file
! Qestring           -> Print evolution of the string
! ustring            -> Unit for evolution of the string file
! ufe                -> Unit for free energy along a given path
! ctgauss            -> Constant factor to pass from bias potential to FES
! Qangcv             -> Logical variable which indicates if there are angle CVs
! angcv(ncv)         -> Logical variable which indicates if a given CV is an angle
! dcv(ncv)           -> Constant norm for angle CVs
! ngrid(ncv)         -> Number of grid points in CV space for FES estimator  
! cvlim(2,ncv)       -> System limits for FES estimator
! cvpoint(ncv)       -> CV point for FES estimator
! ufel               -> Unit for FES estimator
! dt                 -> BD time step
! kbt                -> Thermal energy
! ikbt               -> Inverse of thermal energy
! Qgauss             -> FES and its gradient from Gaussian hills file
! QBspline           -> FES and its gradient from B-spline interpolation (ncv = 2, 3)
! Qbicubic           -> FES and its gradient from bicubic interpolation (ncv = 2)
! ng(ncv)            -> Number of grid points for each CV direction
! ng2(ncv)           -> New number of grid points for each CV direction
! dcel(ncv)          -> Grid size for each CV direction
! idcel(ncv)         -> Inverse of the grid size for each CV direction
! mxgrid             -> Maximum number of grid points for a given CV direction
! xg(mxgrid,ncv)     -> Grid points for each CV direction
! fesg(:)            -> FES for each grip point in CV space
! ugrid              -> Unit for FES defined at CV grid points
! ugrid2             -> Unit for FES defined at CV new grid points
! cg(4,4,:)          -> Coefficients for the bicubic interpolation

module ftsmod
implicit none
integer ncv, mig, mxgrid
integer ngauss, ngskip, nigauss
integer ftsit, nfts, nftskip
integer uftsm, uftsm2, ufts, ufts2, ugauss, ugrid, ugrid2
integer utube, ustring, ufe, ufel
integer, allocatable :: ngrid(:), ng(:), ng2(:)
real, allocatable :: Mm(:,:), Um(:,:), lamb(:), zrep(:,:), rho(:,:), Mminv(:,:), sigmat(:,:)
real, allocatable :: zgauss(:,:), dgauss(:), wgauss(:)
real, allocatable :: Lcur(:), ldist(:), dcv(:)
real, allocatable :: cvlim(:,:), cvpoint(:), dcel(:), idcel(:), xg(:,:), fesg(:), cg(:,:,:)
real ctgauss, kappt, drho, ftstol, kbt, ikbt, dt
logical*1 Qftsm, Qfts, Qconv, Qtube, Qestring, Qangcv, Qgauss, QBspline, Qbicubic
logical*1, allocatable :: angcv(:)

end module
