SUBROUTINE matrices
! Get inverse matric tensor and random tensor
use ftsmod
use errormod
use stdiomod

implicit none
! local variables
integer i, j, k
character :: wrtline*2048

! I. Check proper values for eigenvalues
do i = 1, ncv
  if (lamb(i)<=0.0) then
     write(wrtline,*) 'CV = ',i,'; eigenvalue = ',lamb(i)
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     call error('matrices', 'Wrong eigenvalues', faterr)
  endif
enddo

! II. Inverse metric tensor
do i = 1, ncv
  do j = i, ncv
    Mminv(i,j) = 0.0
    do k = 1, ncv
      Mminv(i,j) = Mminv(i,j) + Um(i,k)*Um(j,k)/lamb(k)
    enddo
    Mminv(j,i) = Mminv(i,j)
  enddo
enddo

! III. Random tensor
do i = 1, ncv
  do j = 1, ncv
    sigmat(i,j) = Um(i,j)*sqrt(lamb(j))
  enddo
enddo

! IV. Print matrices file
if (Qftsm) then
  write(wrtline,*) '! Metric tensor:'
  write(uftsm,'(a)') trim(adjustl(wrtline))
  do i = 1, ncv
    write(wrtline,*) (Mm(i,j),j=1,ncv)
    write(uftsm,'(a)') trim(adjustl(wrtline))
  enddo 
  write(wrtline,*) '! Inverse metric tensor:'
  write(uftsm,'(a)') trim(adjustl(wrtline))
  do i = 1, ncv
    write(wrtline,*) (Mminv(i,j),j=1,ncv)
    write(uftsm,'(a)') trim(adjustl(wrtline))
  enddo 
  write(wrtline,*) '! Random tensor:'
  write(uftsm,'(a)') trim(adjustl(wrtline))
  do i = 1, ncv
    write(wrtline,*) (sigmat(i,j),j=1,ncv)
    write(uftsm,'(a)') trim(adjustl(wrtline))
  enddo 
endif
  
return
END SUBROUTINE matrices
