real FUNCTION pmf(i,l)
! Free Energy Surface (FES)
! Note: dgauss = 1.0/dgauss**2
!       ctgauss includes number of states for computing the mean value
use ftsmod
use errormod

implicit none
! input variables
integer i, l
! local variables
integer j, k, ll
integer ixmax, iymax, izmax, ix, iy, iz
integer nx, ny, nz, ipx, ipy, ipz
real tmp(ncv), theta(ncv), expval, biasV
real x, y, z, vab1, vab2
real mx, my, mz, m2
real t, u

! ** Transformation from cartesian to angle CVs:
if (l==1) then ! FTSM order
  do j = 1, ncv
    tmp(j) = rho(j,i)
  enddo
else ! l=2, STRM order
  do j = 1, ncv
    tmp(j) = zrep(j,i)
  enddo
endif
do j = 1, ncv
  theta(j) = tmp(j)
  if (Qangcv) then
    if (angcv(j)) theta(j) = acos(tmp(j)/dcv(j)) 
  endif   
enddo
! ** Computation of FES:
if (Qgauss) then ! from Gaussian hills file
  biasV = 0.0
  do k = 1, nigauss
    expval = 0.0
    do j = 1, ncv
      expval = expval - (theta(j)-zgauss(j,k))**2*dgauss(j)*0.5
    enddo
    biasV = biasV + wgauss(k)*exp(expval)
  enddo
  pmf = -biasV
  k = nigauss
  do while (k<ngauss)
    k = k + 1
    expval = 0.0
    do j = 1, ncv
      expval = expval - (theta(j)-zgauss(j,k))**2*dgauss(j)*0.5
    enddo
    biasV = biasV + wgauss(k)*exp(expval)
    pmf = pmf - biasV
  enddo
  pmf = ctgauss*pmf
else if (QBspline) then ! B-spline interpolation
  pmf = 0.0 
  ixmax = ng(1)
  iymax = ng(2)
  if (theta(1)<xg(1,1) .or. theta(1)>xg(ixmax,1)) call error ('pmf', 'Point is out of the CV grid', faterr)
  if (theta(2)<xg(1,2) .or. theta(2)>xg(iymax,2)) call error ('pmf', 'Point is out of the CV grid', faterr)
  ! CV coordinates in the local grid system
  x = theta(1) - xg(1,1)
  y = theta(2) - xg(1,2)
  ! integer counter for CV coordinates   
  ix = nint(x*idcel(1))
  iy = nint(y*idcel(2))
  if (ix.eq.0) ix = 1
  if (iy.eq.0) iy = 1
  if (ix.eq.(ixmax-1)) ix = ixmax - 2
  if (iy.eq.(iymax-1)) iy = iymax - 2
  if (ncv==2) then
    ! distribution by 9 adjacent grid points
    do nx = ix-1, ix+1
      ipx = nx*iymax
      vab1 = 1.5 + (nx*dcel(1)-x)*idcel(1)
      vab2 = vab1 - 1.0
      mx = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
      do ny = iy-1, iy+1
        ipy = ny + ipx + 1
        vab1 = 1.5 + (ny*dcel(2)-y)*idcel(2)
        vab2 = vab1 - 1.0
        my = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
        pmf = pmf + mx*my*fesg(ipy)  
      enddo
    enddo
  else ! ncv=3
    izmax = ng(3)
    if (theta(3)<xg(1,3) .or. theta(3)>xg(izmax,3)) call error ('gradpmf', 'Point is out of the CV grid', faterr)
    ! CV coordinates in the local grid system
    z = theta(3) - xg(1,3)
    ! integer counter for CV coordinates   
    iz = nint(z*idcel(3))
    if (iz.eq.0) iz = 1
    if (iz.eq.(izmax-1)) iz = izmax - 2
    ! distribution by 27 adjacent grid points
    do nx = ix-1, ix+1
      ipx = nx*iymax*izmax
      vab1 = 1.5 + (nx*dcel(1)-x)*idcel(1)
      vab2 = vab1 - 1.0
      mx = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
      do ny = iy-1, iy+1
        ipy = ny*izmax + ipx
        vab1 = 1.5 + (ny*dcel(2)-y)*idcel(2)
        vab2 = vab1 - 1.0
        my = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
        do nz = iz-1, iz+1
          ipz = nz + ipy + 1
          vab1 = 1.5 + (nz*dcel(3)-z)*idcel(3)
          vab2 = vab1 - 1.0
          mz = 0.5*(vab1*m2(vab1)+(3.0-vab1)*m2(vab2))
          pmf = pmf + mx*my*mz*fesg(ipz)
        enddo
      enddo
    enddo
  endif
elseif (Qbicubic) then ! bicubic interpolation
  if (theta(1)<xg(1,1) .or. theta(1)>xg(ng(1),1)) call error ('pmf', 'Point is out of the CV grid', faterr)
  if (theta(2)<xg(1,2) .or. theta(2)>xg(ng(2),2)) call error ('pmf', 'Point is out of the CV grid', faterr)
  ! CV coordinates in the local grid system
  x = theta(1) - xg(1,1)
  y = theta(2) - xg(1,2)
  ! integer counter for CV coordinates and coefficients for the bicubic interpolation
  j = int(x*idcel(1)) + 1
  k = int(y*idcel(2)) + 1
  ll = (ng(1)-1)*(k-1) + j
  ! interpolated function value
  t = (theta(1)-xg(j,1))*idcel(1)
  u = (theta(2)-xg(k,2))*idcel(2)
  pmf = 0.0
  do nx = 4, 1, -1
    pmf = t*pmf + ((cg(nx,4,ll)*u+cg(nx,3,ll))*u + cg(nx,2,ll))*u + cg(nx,1,ll)
  enddo
endif

END FUNCTION pmf
