subroutine shell_simul
use ioxmod
use stdiomod
use constamod
use errormod
use charfuncmod   !command parser
use rngmod
use ftsmod

implicit none
! Local variables
character com*2048,fnam*80,wrd5*5,title*80,wrtline*2048
integer unvec(maxopen)
logical*1 logfinal
!for time
integer*8    timer,start,finish
character*8  date
character*10 time
character*5  zone
integer   values(8)
integer   iseed1,iseed2
integer i,j,ij
integer iunit,lfnam
! String method
logical*1 Qmetric, Qrmat, Qgmat, Qfts2, Qrimag, Qgimag, Qftsfe
real temp
real, allocatable :: Mm2(:,:)

!Default parameters
!------------------
unvec        = -1

call header(outu)
start = timer()
call DATE_AND_TIME(date, time, zone, values)
write(outu,'(1x,a,1x,i2,a,i2,a,i4,/,1x,a,1x,i2,a,i2,a,i2)') 'day, month and year (DD/MM/YYYY): ', &
    values(3),'/',values(2),'/',values(1),'hours, minutes and seconds (HH:MM.SS): ',values(5),':',values(6),'.',values(7)
write(outu,*)
write(outu,*)

logfinal = .false. 
do while (.not. logfinal)
  call getlin(com,inpu,outu) ! new commands line
  write(outu,'(/a)')'**********************************************'
  write(outu,'(a)') trim(adjustl(com))
  write(outu,'(a/)')'**********************************************'
  call getfirst(com,wrd5) ! new key word
  wrd5=lcase(wrd5)

  !.....MISCELLANEOUS COMMANDS    
  if  (wrd5.eq.'title') then
  !    ---------------
     call getwrd(com,'name',title)

     write(outu,'(6x,a)') 'TITLE: '//trim(title)
     write(outu,*)

  ! **********************************************************************
  elseif (wrd5.eq.'open')then
  !       --------------
     ! unit [integer,default=1]
     call gtipar(com,'unit',iunit,1)
     if (iunit.le.0) call error ('shell_simul', 'Negative unit in OPEN order', faterr)
     if (iunit.gt.maxopen) call error ('shell_simul', 'Unit in greather than maxopen in OPEN order', faterr)
     if (unvec(iunit).ne.-1) call error ('shell_simul', 'Unit is not available in OPEN order',faterr)
     call lualloc(unvec(iunit))
     iunit = unvec(iunit)
     ! namefile [character]
     call getwrd(com,'name',fnam)
     lfnam=len_trim(fnam)
     write(wrtline,*) 'open file ',fnam(1:lfnam),' as unit ',iunit
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     write(outu,*) 
     ! write/ read options
     if (check(com,'write')) then
       if (check(com,'file')) then ! binary file
         open(unit=iunit,file=fnam,form='unformatted')
       else
         open(unit=iunit,file=fnam,form='formatted')
       endif
     elseif (check(com,'read')) then
       if (check(com,'file')) then
         open(unit=iunit,file=fnam,status='old',form='unformatted')
         rewind(unit=iunit) ! rewinds a file to the beginning
       else
         open(unit=iunit,file=fnam,status='old',form='formatted')
         rewind(unit=iunit) ! rewinds a file to the beginning
       endif
     endif

  ! **********************************************************************
  elseif (wrd5.eq.'close') then
  !       ---------------
     ! unit [integer,default=1]
     call gtipar(com,'unit',iunit,1)
     if (iunit.le.0 .or. iunit.gt.maxopen) call error ('shell_simul', 'Wrong unit in CLOSE order', faterr)
     if (unvec(iunit).eq.-1) call error ('shell_simul', 'Wrong unit in CLOSE order', faterr)
     ij = unvec(iunit) 
     unvec(iunit) = -1
     iunit = ij
     close(unit=iunit)
     call lunalloc (iunit)

     write(wrtline,*) 'close unit',iunit
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     write(outu,*)

  ! **********************************************************************
  elseif (wrd5.eq.'ftsm') then
  !       --------------
     ! Module I: METRIC TENSOR (AND INVERSE) AND RANDOM TENSOR 
     write(wrtline,*) 'METRIC TENSOR (AND INVERSE) AND RANDOM TENSOR'
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     write(wrtline,*) '---------------------------------------------'
     write(outu,'(6x,a)') trim(adjustl(wrtline)) 
     ! Read/generate matrices
     Qrmat = check(com,'rmat')
     Qgmat = check(com,'gmat')
     if ((Qrmat.and.Qgmat) .or. (.not.Qrmat.and..not.Qgmat)) call error ('shell_simul', 'Wrong obtention of matrices in FTSM order', faterr)
     ! Number of collective variables (CVs) [integer, default=0]
     call gtipar(com,'ncv',ncv,0)
     if (ncv<=0) call error ('shell_simul', 'Wrong number of CVs in FTSM order', faterr)
     write(wrtline,*) 'Number of CVs: ',ncv
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     if (allocated(Mm)) deallocate(Mm)
     if (allocated(Mminv)) deallocate(Mminv)
     if (allocated(sigmat)) deallocate(sigmat)
     allocate(Mm(ncv,ncv),Mminv(ncv,ncv),sigmat(ncv,ncv))
     if (Qrmat) then
       ! Unit for reading matrices [integer, default=0]
       call gtipar(com,'munit',uftsm,0)
       if (uftsm<=0 .or. uftsm>maxopen) call error ('shell_simul', 'Wrong unit for reading matrices in FTSM order', faterr)
       if (unvec(uftsm)==-1) call error ('shell_simul', 'Wrong unit for reading matrices in FTSM order', faterr)
       uftsm = unvec(uftsm)
       write(wrtline,*) 'Unit for reading matrices: ',uftsm
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       call readmatrices
     endif
     if (Qgmat) then
       if (allocated(lamb)) deallocate(lamb)
       if (allocated(Um)) deallocate(Um)
       allocate(lamb(ncv),Um(ncv,ncv))
       ! Unit for metric tensor file [integer, default=0]
       call gtipar(com,'imunit',uftsm2,0)
       if (uftsm2<=0 .or. uftsm2>maxopen) call error ('shell_simul', 'Wrong unit for metric tensor file in FTSM order', faterr)
       if (unvec(uftsm2)==-1) call error ('shell_simul', 'Wrong unit for metric tensor file in FTSM order', faterr)
       uftsm2 = unvec(uftsm2)
       write(wrtline,*) 'Unit for metric tensor file: ',uftsm2
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Logical variable which indicates if matrices file is printed
       Qftsm = check(com,'pmat')
       if (Qftsm) then
         ! Unit for printing matrices [integer, default=0]
         call gtipar(com,'munit',uftsm,0)
         if (uftsm<=0 .or. uftsm>maxopen) call error ('shell_simul', 'Wrong unit for printing matrices in FTSM order', faterr)
         if (unvec(uftsm)==-1) call error ('shell_simul', 'Wrong unit for printing matrices in FTSM order', faterr)
         uftsm = unvec(uftsm)
         write(wrtline,*) 'Unit for printing matrices: ',uftsm
         write(outu,'(6x,a)') trim(adjustl(wrtline))
       endif
       call readMm
       allocate(Mm2(ncv,ncv))
       Mm2 = Mm ! Copy Metric tensor before calling jacobi routine     
       call jacobi(Mm2)
       deallocate(Mm2)
       call matrices
     endif
     ! Module II: FINITE TEMPERATURE STRING METHOD
     Qfts2 = check(com,'fts')
     if (Qfts2) then 
       write(wrtline,*) 'FINITE TEMPERATURE STRING METHOD'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       write(wrtline,*) '--------------------------------'
       write(outu,'(6x,a)') trim(adjustl(wrtline))  
       ! Number of images [integer, default=0]   
       call gtipar(com,'mig',mig,0)
       if (mig<=3) call error ('shell_simul', 'Wrong number of images in FTSM order', faterr)
       write(wrtline,*) 'Number of images: ',mig
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Read/generate initial string
       Qrimag = check(com,'rimag')
       Qgimag = check(com,'gimag')
       if ((Qrimag.and.Qgimag) .or. (.not.Qrimag.and..not.Qgimag)) call error ('shell_simul', 'Wrong obtention of initial string in FTSM order', faterr)
       if (allocated(rho)) deallocate(rho)
       if (allocated(zrep)) deallocate(zrep)
       if (allocated(dcv)) deallocate(dcv)
       if (allocated(angcv)) deallocate(angcv)
       allocate(rho(ncv,mig),zrep(ncv,mig),dcv(ncv),angcv(ncv))
       if (Qrimag) then
         ! Unit for reading initial images [integer, default=0]
         call gtipar(com,'ufts',ufts,0)
         if (ufts<=0 .or. ufts>maxopen) call error ('shell_simul', 'Wrong unit for reading initial images in FTSM order', faterr)
         if (unvec(ufts)==-1) call error ('shell_simul', 'Wrong unit for reading initial images in FTSM order', faterr)
         ufts = unvec(ufts)
         write(wrtline,*) 'Unit for reading initial images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         call readimages(1)
       endif
       if (Qgimag) then
        ! Unit for reading initial images [integer, default=0]
         call gtipar(com,'iufts',ufts2,0)
         if (ufts2<=0 .or. ufts2>maxopen) call error ('shell_simul', 'Wrong unit for initial end images in FTSM order', faterr)
         if (unvec(ufts2)==-1) call error ('shell_simul', 'Wrong unit for initial end images in FTSM order', faterr)
         ufts2 = unvec(ufts2)
         write(wrtline,*) 'Unit for initial end images: ',ufts2
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         call readiimag(1)
         call gimages(1)
       endif
       Qgauss = check(com,'gauss')
       QBspline = check(com,'Bspline')
       Qbicubic = check(com,'bicubic')
       if (Qgauss .and. QBspline) call error ('shell_simul', 'Gaussian hills and B-spline can not be invoked simultaneously in FTSM order', faterr)
       if (Qgauss .and. Qbicubic) call error ('shell_simul', 'Gaussian hills and bicubic interpolation can not be invoked simultaneously in FTSM order', faterr)
       if (QBspline .and. Qbicubic) call error ('shell_simul', 'B-spline and  bicubic interpolation can not be invoked simultaneously in FTSM order', faterr)
       if (QBspline .and. (ncv<2.or.ncv>3)) call error ('shell_simul', 'ncv < 2 or ncv > 3 are not valid for B-spline in FTSM order', faterr)
       if (Qbicubic .and. ncv/=2) call error ('shell_simul', 'ncv /= 2 is not valid for bicubic interpolation in FTSM order', faterr)
       if (.not.Qgauss .and. .not.QBspline .and. .not.Qbicubic) call error ('shell_simul', 'Gaussian hills, B-spline and bicubic interpolation are not invoked in FTSM order', faterr)
       if (Qgauss) then
         write(outu,'(6x,a)') 'Gaussian hills for estimating the FES'
       else if (QBspline) then
         write(outu,'(6x,a)') 'B-spline interpolation for estimating the FES'
       else ! Qbicubic
         write(outu,'(6x,a)') 'Bicubic interpolation for estimating the FES'
       endif
       if (Qgauss) then
         ! Unit for reading Gaussian hills [integer, default=0]
         call gtipar(com,'ugauss',ugauss,0)
         if (ugauss<=0 .or. ugauss>maxopen) call error ('shell_simul', 'Wrong unit for Gaussian hills in FTSM order', faterr)
         if (unvec(ugauss)==-1) call error ('shell_simul', 'Wrong unit for Gaussian hills in FTSM order', faterr)
         ugauss = unvec(ugauss)
         write(wrtline,*) 'Unit for Gaussian hills: ',ugauss
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Number of Gaussian functions [integer, default=0]
         call gtipar(com,'ngauss',ngauss,0)
         if (ngauss<=0) call error ('shell_simul', 'Wrong number of Gaussians functions in FTSM order', faterr)
         write(wrtline,*) 'Number of Gaussian functions: ',ngauss
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Number of lines to skip in the hills file [integer, default=0]
         call gtipar(com,'ngskip',ngskip,0)
         if (ngskip<0) call error ('shell_simul', 'Wrong number of lines to skip in the hills file in FTSM order', faterr)
         write(wrtline,*) 'Number of lines to skip in the hills file: ',ngskip
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Constant factor to pass from bias potential to FES [real, default=1.0]
         call gtdpar(com,'ctgauss',ctgauss,1.0)
         write(wrtline,*) 'Constant factor to pass from bias potential to FES: ',ctgauss
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Initial Gaussians for average FES [integer, default=ngauss]
         call gtipar(com,'nigauss',nigauss,ngauss)
         if (nigauss<=0 .or. nigauss>ngauss) call error ('shell_simul', 'Wrong number of initial Gaussians functions in FTSM order', faterr)
         if (nigauss<ngauss) then
           write(wrtline,*) 'Initial Gaussians for average FES: ',nigauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ctgauss = ctgauss/(ngauss-nigauss) ! ctgauss includes number of states for computing mean FES 
         endif
         if (allocated(zgauss)) deallocate(zgauss)
         if (allocated(dgauss)) deallocate(dgauss)
         if (allocated(wgauss)) deallocate(wgauss)
         allocate(zgauss(ncv,ngauss),dgauss(ncv),wgauss(ngauss))
         call readgauss
       endif
       if (QBspline .or. Qbicubic) then
         ! Unit for reading CV grid points [integer, default=0]
         call gtipar(com,'ugrid',ugrid,0)
         if (ugrid<=0 .or. ugrid>maxopen) call error ('shell_simul', 'Wrong unit for CV grid points in FTSM order', faterr)
         if (unvec(ugrid)==-1) call error ('shell_simul', 'Wrong unit for CV grid points in FTSM order', faterr)
         ugrid = unvec(ugrid)
         write(wrtline,*) 'Unit for FES: ',ugrid
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         if (allocated(ng)) deallocate(ng)
         allocate(ng(ncv))
         ! Number of grid points in first CV direction [integer, default=0]
         call gtipar(com,'ng1',ng(1),0)
         if (ng(1)<=0) call error ('shell_simul', 'Wrong number of grid points in first CV direction in FTSM order', faterr)
         mxgrid = ng(1)
         write(wrtline,*) 'Number of grid points in first CV direction: ',ng(1)
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Number of grid points in second CV direction [integer, default=0]
         call gtipar(com,'ng2',ng(2),0)
         if (ng(2)<=0) call error ('shell_simul', 'Wrong number of grid points in second CV direction in FTSM order', faterr)
         if (ng(2)>mxgrid) mxgrid = ng(2)
         write(wrtline,*) 'Number of grid points in second CV direction; ',ng(2)
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         if (ncv==3) then
           ! Number of grid points in third CV direction [integer, default=0]
           call gtipar(com,'ng3',ng(3),0)
           if (ng(3)<=0) call error ('shell_simul', 'Wrong number of grid points in third CV direction in FTSM order', faterr) 
           if (ng(3)>mxgrid) mxgrid = ng(3)         
           write(wrtline,*) 'Number of grid points in third CV direction: ',ng(3)
           write(outu,'(6x,a)') trim(adjustl(wrtline))
         endif
         if (allocated(dcel)) deallocate(dcel)
         if (allocated(idcel)) deallocate(idcel)
         if (allocated(xg)) deallocate(xg)
         if (allocated(fesg)) deallocate(fesg)
         allocate(dcel(ncv),idcel(ncv),xg(mxgrid,ncv))
         if (ncv==2) then
           allocate(fesg(ng(1)*ng(2)))
         else ! ncv = 3
           allocate(fesg(ng(1)*ng(2)*ng(3)))
         endif
         call readgrid
         if (Qbicubic) then
           if (allocated(cg)) deallocate(cg)
           allocate(cg(4,4,(ng(1)-1)*(ng(2)-1)))
           call bsplinec
         endif
       endif
       ! Temperature [real,default=300K]
       call gtdpar(com,'temp',temp,300.0)
       if (temp<0.0) call error ('shell_simul', 'Temperature must be positive in FTSM order',faterr)
       write(wrtline,*)  'Temperature [K]: ',temp
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       kbt = kboltz*temp*avogadro*0.001
       write(wrtline,*) 'Thermal energy [KJ/mole]: ',kbt
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! BD time step [real,default=0.0001]
       call gtdpar(com,'dt',dt,0.0001)
       if (dt<=0.0) call error ('shell_simul', 'Wrong BD time step in FTSM order', faterr)
       write(wrtline,*) 'BD time step [ps]: ',dt
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Random number seeds [integer,default=12345]
       call gtipar(com,'iseed1',iseed1,12345)
       if (iseed1.le.0 .or. iseed1.ge.m1) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       call gtipar(com,'iseed2',iseed2,12345)
       if (iseed2.le.0 .or. iseed2.ge.m2) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       call gtdpar(com,'s10',s10,real(iseed1))
       if (s10.lt.0.0 .or. s10.ge.m1 .or. (s10-int(s10)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       call gtdpar(com,'s11',s11,real(iseed1))
       if (s11.lt.0.0 .or. s11.ge.m1 .or. (s11-int(s11)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       call gtdpar(com,'s12',s12,real(iseed1))
       if (s12.lt.0.0 .or. s12.ge.m1 .or. (s12-int(s12)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       call gtdpar(com,'s20',s20,real(iseed2))
       if (s20.lt.0.0 .or. s20.ge.m2 .or. (s20-int(s20)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       call gtdpar(com,'s21',s21,real(iseed2))
       if (s21.lt.0.0 .or. s21.ge.m2 .or. (s21-int(s21)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       call gtdpar(com,'s22',s22,real(iseed2))
       if (s22.lt.0.0 .or. s22.ge.m2 .or. (s22-int(s22)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
       if (int(s10).eq.0 .and. int(s11).eq.0 .and. int(s12).eq.0) call error ('shell_simul', 'Wrong seed numbers in SYSTEM order', faterr)
       if (int(s20).eq.0 .and. int(s21).eq.0 .and. int(s22).eq.0) call error ('shell_simul', 'Wrong seed numbers in SYSTEM order', faterr)
       write(wrtline,*) 'Using seeds: ',s10,s11,s12,s20,s21,s22
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! String step [real, default=0.1]
       call gtdpar(com,'drho',drho,0.1)
       if (drho<=0.0) call error ('shell_simul', 'Wrong images time step in FTSM order', faterr)
       write(wrtline,*) 'Images time step: ',drho
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Smoothing parameter [real, default=0.1]
       call gtdpar(com,'kappa',kappt,0.1)
       if (kappt<0.0) call error ('shell_simul', 'Smoothing parameter is lower or equal to zero in FTSM order', faterr)
       kappt = kappt*mig*drho
       write(wrtline,*) 'Smoothing parameter: ',kappt
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Maximum number of iterations [integer, default=10000]
       call gtipar(com,'ftsit',ftsit,10000)
       if (ftsit<1) call error ('shell_simul', 'Wrong maximum number of iterations in FTSM order', faterr)
       write(wrtline,*) 'Maximum nuber of iterations: ',ftsit
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Logical variable which indicates if images file is printed
       Qfts = check(com,'pimag')
       if (Qfts .and. .not.Qrimag) then
         ! Unit for printing final images [integer, default=0]
         call gtipar(com,'ufts',ufts,0)
         if (ufts<=0 .or. ufts>maxopen) call error ('shell_simul', 'Wrong unit for printing final images in FTSM order', faterr)
         if (unvec(ufts)==-1) call error ('shell_simul', 'Wrong unit for printing final images in FTSM order', faterr)
         ufts = unvec(ufts)
         write(wrtline,*) 'Unit for printing final images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
       endif
       if (Qfts .and. Qrimag) then
         call error ('shell_simul', 'File for initial images is also used for the final images', warning)
         write(wrtline,*) 'Unit for printing final images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline)) 
         rewind(unit=ufts) ! rewinds a file to the beginning
       endif
       ! Logical variable which indicates if evolution of the string is printed 
       Qestring = check(com,'evolution')
       if (Qestring) then
         ! Unit for evolution of the string file [integer, default=0]
         call gtipar(com,'ustring',ustring,0)
         if (ustring<=0 .or. ustring>maxopen) call error ('shell_simul', 'Wrong unit for string file in FTSM order', faterr)
         if (unvec(ustring)==-1) call error ('shell_simul', 'Wrong unit for string file in FTSM order', faterr)
         ustring = unvec(ustring)
         write(wrtline,*) 'Unit for evolution of the string file: ',ustring
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Evolution of the string file:'
         write(ustring,'(a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Number of CVs: ',ncv
         write(ustring,'(a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Number of images: ',mig
         write(ustring,'(a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Iteration --- euclidean distance from the initial condition'
         write(ustring,'(a)') trim(adjustl(wrtline))
       endif
       ! Check convergence
       Qconv = check(com,'conv')
       if (Qconv) then
         ! Tolerance [real, default=max(mig^(-4),10^(-10))]
         call gtdpar(com,'ftstol',ftstol,max((1.0*mig)**(-4),1.0e-10))
         if (ftstol<=0.0) call error ('shell_simul', 'Wrong tolerance value for convergence in FTSM order', faterr)
         write(wrtline,*) 'Tolerance value for convergence: ',ftstol
         write(outu,'(6x,a)') trim(adjustl(wrtline)) 
       endif
       if (allocated(Lcur)) deallocate(Lcur)
       if (allocated(ldist)) deallocate(ldist)
       allocate(Lcur(mig),ldist(mig)) 
       call fts
       write(wrtline,*) 'Final images for the string:'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       do i = 1, mig
         write(wrtline,*) (rho(j,i), j=1,ncv)
         write(outu,'(6x,a)') trim(adjustl(wrtline))
       enddo
     endif
     ! Module III: FREE ENERGY ALONG A GIVEN PATH
     Qftsfe = check(com,'ftsfe')
     if (Qftsfe) then
       write(wrtline,*) 'FREE ENERGY ALONG A GIVEN PATH'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       write(wrtline,*) '------------------------------'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       if (.not.Qfts2) then
         ! Number of images [integer, default=0]   
         call gtipar(com,'mig',mig,0)
         if (mig<=3) call error ('shell_simul', 'Wrong number of images in FTSM order', faterr)
         write(wrtline,*) 'Number of images: ',mig
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         if (allocated(rho)) deallocate(rho)
         if (allocated(zrep)) deallocate(zrep)
         if (allocated(dcv)) deallocate(dcv)
         if (allocated(angcv)) deallocate(angcv)
         allocate(rho(ncv,mig),zrep(ncv,mig),dcv(ncv),angcv(ncv))
         ! Unit for reading initial images [integer, default=0]
         call gtipar(com,'ufts',ufts,0)
         if (ufts<=0 .or. ufts>maxopen) call error ('shell_simul', 'Wrong unit for reading initial images in FTSM order', faterr)
         if (unvec(ufts)==-1) call error ('shell_simul', 'Wrong unit for reading initial images in FTSM order', faterr)
         ufts = unvec(ufts)
         write(wrtline,*) 'Unit for reading initial images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         call readimages(1)
         Qgauss = check(com,'gauss')
         QBspline = check(com,'Bspline')
         Qbicubic = check(com,'bicubic')
         if (Qgauss .and. QBspline) call error ('shell_simul', 'Gaussian hills and B-spline can not be invoked simultaneously in FTSM order', faterr)
         if (Qgauss .and. Qbicubic) call error ('shell_simul', 'Gaussian hills and bicubic interpolation can not be invoked simultaneously in FTSM order', faterr)
         if (QBspline .and. Qbicubic) call error ('shell_simul', 'B-spline and  bicubic interpolation can not be invoked simultaneously in FTSM order', faterr)
         if (QBspline .and. (ncv<2.or.ncv>3)) call error ('shell_simul', 'ncv < 2 or ncv > 3 is not valid for B-spline in FTSM order', faterr)
         if (Qbicubic .and. ncv/=2) call error ('shell_simul', 'ncv /= 2 is not valid for bicubic interpolation in FTSM order', faterr)
         if (.not.Qgauss .and. .not.QBspline .and. .not.Qbicubic) call error ('shell_simul', 'Gaussian hills, B-spline and bicubic interpolation are not invoked in FTSM order', faterr)
         if (Qgauss) then
           write(outu,'(6x,a)') 'Gaussian hills for estimating the FES'
         else if (QBspline) then
           write(outu,'(6x,a)') 'B-spline interpolation for estimating the FES'
         else ! Qbicubic
           write(outu,'(6x,a)') 'Bicubic interpolation for estimating the FES'
         endif
         if (Qgauss) then
           ! Unit for reading Gaussian hills [integer, default=0]
           call gtipar(com,'ugauss',ugauss,0)
           if (ugauss<=0 .or. ugauss>maxopen) call error ('shell_simul', 'Wrong unit for Gaussian hills in FTSM order', faterr)
           if (unvec(ugauss)==-1) call error ('shell_simul', 'Wrong unit for Gaussian hills in FTSM order', faterr)
           ugauss = unvec(ugauss)
           write(wrtline,*) 'Unit for Gaussian hills: ',ugauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Number of Gaussian functions [integer, default=0]
           call gtipar(com,'ngauss',ngauss,0)
           if (ngauss<=0) call error ('shell_simul', 'Wrong number of Gaussians functions in FTSM order', faterr)
           write(wrtline,*) 'Number of Gaussian functions: ',ngauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Number of lines to skip in the hills file [integer, default=0]
           call gtipar(com,'ngskip',ngskip,0)
           if (ngskip<0) call error ('shell_simul', 'Wrong number of lines to skip in the hills file in FTSM order', faterr)
           write(wrtline,*) 'Number of lines to skip in the hills file: ',ngskip
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Constant factor to pass from bias potential to FES [real, default=1.0]
           call gtdpar(com,'ctgauss',ctgauss,1.0)
           write(wrtline,*) 'Constant factor to pass from bias potential to FES: ',ctgauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Initial Gaussians for average FES [integer, default=ngauss]
           call gtipar(com,'nigauss',nigauss,ngauss)
           if (nigauss<=0 .or. nigauss>ngauss) call error ('shell_simul', 'Wrong number of initial Gaussians functions in FTSM order', faterr)
           if (nigauss<ngauss) then
             write(wrtline,*) 'Initial Gaussians for average FES: ',nigauss
             write(outu,'(6x,a)') trim(adjustl(wrtline))
             ctgauss = ctgauss/(ngauss-nigauss) ! ctgauss includes number of states for computing mean FES 
           endif
           if (allocated(zgauss)) deallocate(zgauss)
           if (allocated(dgauss)) deallocate(dgauss)
           if (allocated(wgauss)) deallocate(wgauss)
           allocate(zgauss(ncv,ngauss),dgauss(ncv),wgauss(ngauss))
           call readgauss
         endif
         if (QBspline .or. Qbicubic) then
           ! Unit for reading CV grid points [integer, default=0]
           call gtipar(com,'ugrid',ugrid,0)
           if (ugrid<=0 .or. ugrid>maxopen) call error ('shell_simul', 'Wrong unit for CV grid points in FTSM order', faterr)
           if (unvec(ugrid)==-1) call error ('shell_simul', 'Wrong unit for CV grid points in FTSM order', faterr)
           ugrid = unvec(ugrid)
           write(wrtline,*) 'Unit for FES: ',ugrid
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           if (allocated(ng)) deallocate(ng)
           allocate(ng(ncv))
           ! Number of grid points in first CV direction [integer, default=0]
           call gtipar(com,'ng1',ng(1),0)
           if (ng(1)<=0) call error ('shell_simul', 'Wrong number of grid points in first CV direction in FTSM order', faterr)
           mxgrid = ng(1)
           write(wrtline,*) 'Number of grid points in first CV direction: ',ng(1)
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Number of grid points in second CV direction [integer, default=0]
           call gtipar(com,'ng2',ng(2),0)
           if (ng(2)<=0) call error ('shell_simul', 'Wrong number of grid points in second CV direction in FTSM order', faterr)
           if (ng(2)>mxgrid) mxgrid = ng(2)
           write(wrtline,*) 'Number of grid points in second CV direction: ',ng(2)
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           if (ncv==3) then
             ! Number of grid points in third CV direction [integer, default=0]
             call gtipar(com,'ng3',ng(3),0)
             if (ng(3)<=0) call error ('shell_simul', 'Wrong number of grid points in third CV direction in FTSM order', faterr) 
             if (ng(3)>mxgrid) mxgrid = ng(3)         
             write(wrtline,*) 'Number of grid points in third CV direction: ',ng(3)
             write(outu,'(6x,a)') trim(adjustl(wrtline))
           endif
           if (allocated(dcel)) deallocate(dcel)
           if (allocated(idcel)) deallocate(idcel)
           if (allocated(xg)) deallocate(xg)
           if (allocated(fesg)) deallocate(fesg)
           allocate(dcel(ncv),idcel(ncv),xg(mxgrid,ncv))
           if (ncv==2) then
             allocate(fesg(ng(1)*ng(2)))
           else ! ncv = 3
             allocate(fesg(ng(1)*ng(2)*ng(3)))
           endif
           call readgrid
           if (Qbicubic) then 
             if (allocated(cg)) deallocate(cg)
             allocate(cg(4,4,(ng(1)-1)*(ng(2)-1)))
             call bsplinec
           endif
         endif
       endif
       ! Unit for free energy along a given path [integer, default=0]
       call gtipar(com,'ufe',ufe,0)
       if (ufe<=0 .or. ufe>maxopen) call error ('shell_simul', 'Wrong unit for free energy along a given path in FTSM order', faterr)
       if (unvec(ufe)==-1) call error ('shell_simul', 'Wrong unit for free energy along a given path in FTSM order', faterr)
       ufe = unvec(ufe)
       write(wrtline,*) 'Unit for free energy along a given path: ',ufe
       write(outu,'(6x,a)') trim(adjustl(wrtline)) 
       ! Logical variable which indicates if reaction tube is printed
       Qtube = check(com,'tube')
       if (Qtube) then
         if (.not.Qfts2) then
           ! Temperature [real,default=300K]
           call gtdpar(com,'temp',temp,300.0)
           if (temp<0.0) call error ('shell_simul', 'Temperature must be positive in FTSM order',faterr)
           write(wrtline,*)  'Temperature [K]: ',temp
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           kbt = kboltz*temp*avogadro*0.001
           write(wrtline,*) 'Thermal energy [KJ/mole]: ',kbt
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! BD time step [real,default=0.0001]
           call gtdpar(com,'dt',dt,0.0001)
           if (dt<=0.0) call error ('shell_simul', 'Wrong BD time step in FTSM order', faterr)
           write(wrtline,*) 'BD time step: ',dt
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Random number seeds [integer,default=12345]
           call gtipar(com,'iseed1',iseed1,12345)
           if (iseed1.le.0 .or. iseed1.ge.m1) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           call gtipar(com,'iseed2',iseed2,12345)
           if (iseed2.le.0 .or. iseed2.ge.m2) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           call gtdpar(com,'s10',s10,real(iseed1))
           if (s10.lt.0.0 .or. s10.ge.m1 .or. (s10-int(s10)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           call gtdpar(com,'s11',s11,real(iseed1))
           if (s11.lt.0.0 .or. s11.ge.m1 .or. (s11-int(s11)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           call gtdpar(com,'s12',s12,real(iseed1))
           if (s12.lt.0.0 .or. s12.ge.m1 .or. (s12-int(s12)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           call gtdpar(com,'s20',s20,real(iseed2))
           if (s20.lt.0.0 .or. s20.ge.m2 .or. (s20-int(s20)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           call gtdpar(com,'s21',s21,real(iseed2))
           if (s21.lt.0.0 .or. s21.ge.m2 .or. (s21-int(s21)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           call gtdpar(com,'s22',s22,real(iseed2))
           if (s22.lt.0.0 .or. s22.ge.m2 .or. (s22-int(s22)).gt.0.0) call error ('shell_simul', 'Wrong seed number in SYSTEM order', faterr)
           if (int(s10).eq.0 .and. int(s11).eq.0 .and. int(s12).eq.0) call error ('shell_simul', 'Wrong seed numbers in SYSTEM order', faterr)
           if (int(s20).eq.0 .and. int(s21).eq.0 .and. int(s22).eq.0) call error ('shell_simul', 'Wrong seed numbers in SYSTEM order', faterr)
           write(wrtline,*) 'Using seeds: ',s10,s11,s12,s20,s21,s22
           write(outu,'(6x,a)') trim(adjustl(wrtline))
         endif
         ! Number of time steps [integer, default=10000]
         call gtipar(com,'nfts',nfts,10000)
         if (nfts<1) call error ('shell_simul', 'Wrong number of time steps in FTSM order', faterr)
         write(wrtline,*) 'Number of time steps: ',nfts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Number of iterations after which reaction tube is printed [integer, default=1]
         call gtipar(com,'nftskip',nftskip,1)
         if (nftskip<1) call error ('shell_simul', 'Wrong number of iterations after which reaction tube is printed in FTSM order', faterr)
         write(wrtline,*) 'Number of iterations skipping (reaction tube): ',nftskip
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Unit for reaction tube file [integer, default=0]
         call gtipar(com,'utube',utube,0)
         if (utube<=0 .or. utube>maxopen) call error ('shell_simul', 'Wrong unit for tube file in FTSM order', faterr)
         if (unvec(utube)==-1) call error ('shell_simul', 'Wrong unit for tube file in FTSM order', faterr)
         utube = unvec(utube)  
         write(wrtline,*) 'Unit for tube file: ',utube
         write(outu,'(6x,a)') trim(adjustl(wrtline)) 
         write(wrtline,*) '# Reaction tube file:'
         write(utube,'(a)') trim(adjustl(wrtline))  
         write(wrtline,*) '# Number of iterations skipping (reaction tube): ',nftskip
         write(utube,'(a)') trim(adjustl(wrtline))   
         write(wrtline,*) '# Number of CVs: ',ncv
         write(utube,'(a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Number of images: ',mig
         write(utube,'(a)') trim(adjustl(wrtline)) 
       endif
       call ftsfe
     endif
     write(outu,*)

  ! **********************************************************************
  elseif (wrd5.eq.'strm') then
  !       --------------
     ! Number of collective variables (CVs) [integer, default=0]
     call gtipar(com,'ncv',ncv,0)
     if (ncv<=0) call error ('shell_simul', 'Wrong number of CVs in STRM order', faterr)
     write(wrtline,*) 'Number of CVs: ',ncv
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Module I: METRIC TENSOR 
     Qmetric = check(com,'metric')
     if (Qmetric) then
       write(wrtline,*) 'METRIC TENSOR'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       write(wrtline,*) '-------------'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Unit for metric tensor file [integer, default=0]
       call gtipar(com,'munit',uftsm2,0)
       if (uftsm2<=0 .or. uftsm2>maxopen) call error ('shell_simul', 'Wrong unit for metric tensor file in STRM order', faterr)
       if (unvec(uftsm2)==-1) call error ('shell_simul', 'Wrong unit for metric tensor file in STRM order', faterr)
       uftsm2 = unvec(uftsm2)
       write(wrtline,*) 'Unit for metric tensor file: ',uftsm2
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       if (allocated(Mm)) deallocate(Mm)
       allocate(Mm(ncv,ncv))
       call readMm
     endif
     ! Module II: STRING METHOD
     Qfts2 = check(com,'string')
     if (Qfts2 .and. .not.Qmetric) call error ('shell_simul', 'Metric tensor has to be defined before string in STRM order', faterr)
     if (Qfts2) then
       write(wrtline,*) 'STRING METHOD'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       write(wrtline,*) '-------------'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Number of images [integer, default=0]   
       call gtipar(com,'mig',mig,0)
       if (mig<=3) call error ('shell_simul', 'Wrong number of images in STRM order', faterr)
       write(wrtline,*) 'Number of images: ',mig
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Read/generate initial string
       Qrimag = check(com,'rimag')
       Qgimag = check(com,'gimag')
       if ((Qrimag.and.Qgimag) .or. (.not.Qrimag.and..not.Qgimag)) call error ('shell_simul', 'Wrong obtention of initial string in STRM order', faterr)
       if (allocated(zrep)) deallocate(zrep)
       if (allocated(dcv)) deallocate(dcv)
       if (allocated(angcv)) deallocate(angcv)
       allocate(zrep(ncv,mig),dcv(ncv),angcv(ncv))
       if (Qrimag) then
         ! Unit for reading initial images [integer, default=0]
         call gtipar(com,'ufts',ufts,0)
         if (ufts<=0 .or. ufts>maxopen) call error ('shell_simul', 'Wrong unit for reading initial images in STRM order', faterr)
         if (unvec(ufts)==-1) call error ('shell_simul', 'Wrong unit for reading initial images in STRM order', faterr)
         ufts = unvec(ufts)
         write(wrtline,*) 'Unit for reading initial images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         call readimages(2)
       endif 
       if (Qgimag) then
        ! Unit for reading initial images [integer, default=0]
         call gtipar(com,'iufts',ufts2,0)
         if (ufts2<=0 .or. ufts2>maxopen) call error ('shell_simul', 'Wrong unit for initial end images in STRM order', faterr)
         if (unvec(ufts2)==-1) call error ('shell_simul', 'Wrong unit for initial end images in STRM order', faterr)
         ufts2 = unvec(ufts2)
         write(wrtline,*) 'Unit for initial end images: ',ufts2
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         call readiimag(2)
         call gimages(2)
       endif      
       Qgauss = check(com,'gauss')
       QBspline = check(com,'Bspline')
       Qbicubic = check(com,'bicubic')
       if (Qgauss .and. QBspline) call error ('shell_simul', 'Gaussian hills and B-spline can not be invoked simultaneously in STRM order', faterr)
       if (Qgauss .and. Qbicubic) call error ('shell_simul', 'Gaussian hills and bicubic interpolation can not be invoked simultaneously in STRM order', faterr)
       if (QBspline .and. Qbicubic) call error ('shell_simul', 'B-spline and  bicubic interpolation can not be invoked simultaneously in STRM order', faterr)
       if (QBspline .and. (ncv<2.or.ncv>3)) call error ('shell_simul', 'ncv < 2 or ncv > 3 is not valid for B-spline in STRM order', faterr)
       if (Qbicubic .and. ncv/=2) call error ('shell_simul', 'ncv /= 2 is not valid for bicubic interpolation in STRM order', faterr)
       if (.not.Qgauss .and. .not.QBspline .and. .not.Qbicubic) call error ('shell_simul', 'Gaussian hills, B-spline and bicubic interpolation are not invoked in STRM order', faterr)
       if (Qgauss) then
         write(outu,'(6x,a)') 'Gaussian hills for estimating the FES'
       else if (QBspline) then
         write(outu,'(6x,a)') 'B-spline interpolation for estimating the FES'
       else ! Qbicubic
         write(outu,'(6x,a)') 'Bicubic interpolation for estimating the FES'
       endif
       if (Qgauss) then
         ! Unit for reading Gaussian hills [integer, default=0]
         call gtipar(com,'ugauss',ugauss,0)
         if (ugauss<=0 .or. ugauss>maxopen) call error ('shell_simul', 'Wrong unit for Gaussian hills in STRM order', faterr)
         if (unvec(ugauss)==-1) call error ('shell_simul', 'Wrong unit for Gaussian hills in STRM order', faterr)
         ugauss = unvec(ugauss)
         write(wrtline,*) 'Unit for Gaussian hills: ',ugauss
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Number of Gaussian functions [integer, default=0]
         call gtipar(com,'ngauss',ngauss,0)
         if (ngauss<=0) call error ('shell_simul', 'Wrong number of Gaussians functions in STRM order', faterr)
         write(wrtline,*) 'Number of Gaussian functions: ',ngauss
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Number of lines to skip in the hills file [integer, default=0]
         call gtipar(com,'ngskip',ngskip,0)
         if (ngskip<0) call error ('shell_simul', 'Wrong number of lines to skip in the hills file in STRM order', faterr)
         write(wrtline,*) 'Number of lines to skip in the hills file: ',ngskip
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Constant factor to pass from bias potential to FES [real, default=1.0]
         call gtdpar(com,'ctgauss',ctgauss,1.0)
         write(wrtline,*) 'Constant factor to pass from bias potential to FES: ',ctgauss
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Initial Gaussians for average FES [integer, default=ngauss]
         call gtipar(com,'nigauss',nigauss,ngauss)
         if (nigauss<=0 .or. nigauss>ngauss) call error ('shell_simul', 'Wrong number of initial Gaussians functions in STRM order', faterr)
         if (nigauss<ngauss) then
           write(wrtline,*) 'Initial Gaussians for average FES: ',nigauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ctgauss = ctgauss/(ngauss-nigauss) ! ctgauss includes number of states for computing mean FES 
         endif 
         if (allocated(zgauss)) deallocate(zgauss)
         if (allocated(dgauss)) deallocate(dgauss)
         if (allocated(wgauss)) deallocate(wgauss)
         allocate(zgauss(ncv,ngauss),dgauss(ncv),wgauss(ngauss))
         call readgauss
       endif
       if (QBspline .or. Qbicubic) then
         ! Unit for reading CV grid points [integer, default=0]
         call gtipar(com,'ugrid',ugrid,0)
         if (ugrid<=0 .or. ugrid>maxopen) call error ('shell_simul', 'Wrong unit for CV grid points in STRM order', faterr)
         if (unvec(ugrid)==-1) call error ('shell_simul', 'Wrong unit for CV grid points in STRM order', faterr)
         ugrid = unvec(ugrid)
         write(wrtline,*) 'Unit for FES: ',ugrid
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         if (allocated(ng)) deallocate(ng)
         allocate(ng(ncv))
         ! Number of grid points in first CV direction [integer, default=0]
         call gtipar(com,'ng1',ng(1),0)
         if (ng(1)<=0) call error ('shell_simul', 'Wrong number of grid points in first CV direction in STRM order', faterr)
         mxgrid = ng(1)
         write(wrtline,*) 'Number of grid points in first CV direction: ',ng(1)
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         ! Number of grid points in second CV direction [integer, default=0]
         call gtipar(com,'ng2',ng(2),0)
         if (ng(2)<=0) call error ('shell_simul', 'Wrong number of grid points in second CV direction in STRM order', faterr)
         if (ng(2)>mxgrid) mxgrid = ng(2)
         write(wrtline,*) 'Number of grid points in second CV direction: ',ng(2)
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         if (ncv==3) then
           ! Number of grid points in third CV direction [integer, default=0]
           call gtipar(com,'ng3',ng(3),0)
           if (ng(3)<=0) call error ('shell_simul', 'Wrong number of grid points in third CV direction in STRM order', faterr) 
           if (ng(3)>mxgrid) mxgrid = ng(3)         
           write(wrtline,*) 'Number of grid points in third CV direction: ',ng(3)
           write(outu,'(6x,a)') trim(adjustl(wrtline))        
         endif
         if (allocated(dcel)) deallocate(dcel)
         if (allocated(idcel)) deallocate(idcel)
         if (allocated(xg)) deallocate(xg)
         if (allocated(fesg)) deallocate(fesg)
         allocate(dcel(ncv),idcel(ncv),xg(mxgrid,ncv))
         if (ncv==2) then
           allocate(fesg(ng(1)*ng(2)))
         else ! ncv = 3
           allocate(fesg(ng(1)*ng(2)*ng(3)))
         endif
         call readgrid
         if (Qbicubic) then 
           if (allocated(cg)) deallocate(cg)
           allocate(cg(4,4,(ng(1)-1)*(ng(2)-1)))
           call bsplinec
         endif
       endif
       ! Time step [real,default=0.0001]
       call gtdpar(com,'dt',dt,0.0001)
       if (dt<=0.0) call error ('shell_simul', 'Wrong time step in STRM order', faterr)
       write(wrtline,*) 'Time step: ',dt
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Smoothing parameter [real, default=0.1]
       call gtdpar(com,'kappa',kappt,0.1)
       if (kappt<0.0 .or. kappt>=1.0) call error ('shell_simul', 'Smoothing parameter is lower or equal to zero in STRM order', faterr)
       write(wrtline,*) 'Smoothing parameter: ',kappt
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Maximum number of iterations [integer, default=10000]
       call gtipar(com,'ftsit',ftsit,10000)
       if (ftsit<1) call error ('shell_simul', 'Wrong maximum number of iterations in STRM order', faterr)
       write(wrtline,*) 'Maximum nuber of iterations: ',ftsit
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       ! Logical variable which indicates if images file is printed
       Qfts = check(com,'pimag')
       if (Qfts .and. .not.Qrimag) then
         ! Unit for printing final images [integer, default=0]
         call gtipar(com,'ufts',ufts,0)
         if (ufts<=0 .or. ufts>maxopen) call error ('shell_simul', 'Wrong unit for printing final images in STRM order', faterr)
         if (unvec(ufts)==-1) call error ('shell_simul', 'Wrong unit for printing final images in STRM order', faterr)
         ufts = unvec(ufts)
         write(wrtline,*) 'Unit for printing final images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
       endif
       if (Qfts .and. Qrimag) then
         call error ('shell_simul', 'File for initial images is also used for the final images', warning)
         write(wrtline,*) 'Unit for printing final images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         rewind(unit=ufts) ! rewinds a file to the beginning
       endif
       ! Logical variable which indicates if evolution of the string is printed 
       Qestring = check(com,'evolution')
       if (Qestring) then
         ! Unit for evolution of the string file [integer, default=0]
         call gtipar(com,'ustring',ustring,0)
         if (ustring<=0 .or. ustring>maxopen) call error ('shell_simul', 'Wrong unit for string file in STRM order', faterr)
         if (unvec(ustring)==-1) call error ('shell_simul', 'Wrong unit for string file in STRM order', faterr)
         ustring = unvec(ustring)
         write(wrtline,*) 'Unit for evolution of the string file: ',ustring
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Evolution of the string file:'
         write(ustring,'(a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Number of CVs: ',ncv
         write(ustring,'(a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Number of images: ',mig
         write(ustring,'(a)') trim(adjustl(wrtline))
         write(wrtline,*) '# Iteration --- euclidean distance from the initial condition'
         write(ustring,'(a)') trim(adjustl(wrtline))
       endif
       ! Check convergence
       Qconv = check(com,'conv')
       if (Qconv) then
         ! Tolerance [real, default=max(mig^(-4),10^(-10))]
         call gtdpar(com,'ftstol',ftstol,max((1.0*mig)**(-4),1.0e-10))
         if (ftstol<=0.0) call error ('shell_simul', 'Wrong tolerance value for convergence in STRM order', faterr)
         write(wrtline,*) 'Tolerance value for convergence: ',ftstol
         write(outu,'(6x,a)') trim(adjustl(wrtline))
       endif
       if (allocated(Lcur)) deallocate(Lcur)
       if (allocated(ldist)) deallocate(ldist)
       allocate(Lcur(mig),ldist(mig))
       call stm
       write(wrtline,*) 'Final images for the string:'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       do i = 1, mig
         write(wrtline,*) (zrep(j,i), j=1,ncv)
         write(outu,'(6x,a)') trim(adjustl(wrtline))
       enddo
     endif
     ! Module III: FREE ENERGY ALONG A GIVEN PATH
     Qftsfe = check(com,'stmfe')
     if (Qftsfe) then
       write(wrtline,*) 'FREE ENERGY ALONG A GIVEN PATH'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       write(wrtline,*) '------------------------------'
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       if (.not.Qfts2) then
         ! Number of images [integer, default=0]   
         call gtipar(com,'mig',mig,0)
         if (mig<=3) call error ('shell_simul', 'Wrong number of images in STRM order', faterr)
         write(wrtline,*) 'Number of images: ',mig
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         if (allocated(zrep)) deallocate(zrep)
         if (allocated(dcv)) deallocate(dcv)
         if (allocated(angcv)) deallocate(angcv)
         allocate(zrep(ncv,mig),dcv(ncv),angcv(ncv))
         ! Unit for reading initial images [integer, default=0]
         call gtipar(com,'ufts',ufts,0)
         if (ufts<=0 .or. ufts>maxopen) call error ('shell_simul', 'Wrong unit for reading initial images in STRM order', faterr)
         if (unvec(ufts)==-1) call error ('shell_simul', 'Wrong unit for reading initial images in STRM order', faterr)
         ufts = unvec(ufts)
         write(wrtline,*) 'Unit for reading initial images: ',ufts
         write(outu,'(6x,a)') trim(adjustl(wrtline))
         call readimages(2)
         Qgauss = check(com,'gauss')
         QBspline = check(com,'Bspline')
         Qbicubic = check(com,'bicubic')
         if (Qgauss .and. QBspline) call error ('shell_simul', 'Gaussian hills and B-spline can not be invoked simultaneously in STRM order', faterr)
         if (Qgauss .and. Qbicubic) call error ('shell_simul', 'Gaussian hills and bicubic interpolation can not be invoked simultaneously in STRM order', faterr)
         if (QBspline .and. Qbicubic) call error ('shell_simul', 'B-spline and  bicubic interpolation can not be invoked simultaneously in STRM order', faterr)
         if (QBspline .and. (ncv<2.or.ncv>3)) call error ('shell_simul', 'ncv < 2 or ncv > 3 are not valid for B-spline in STRM order', faterr)
         if (Qbicubic .and. ncv/=2) call error ('shell_simul', 'ncv /= 2 is not valid for bicubic interpolation in STRM order', faterr)
         if (.not.Qgauss .and. .not.QBspline .and. .not.Qbicubic) call error ('shell_simul', 'Gaussian hills, B-spline and bicubic interpolation are not invoked in STRM order', faterr)
         if (Qgauss) then
           write(outu,'(6x,a)') 'Gaussian hills for estimating the FES'
         else if (QBspline) then
           write(outu,'(6x,a)') 'B-spline interpolation for estimating the FES'
         else ! Qbicubic
           write(outu,'(6x,a)') 'Bicubic interpolation for estimating the FES'
         endif
         if (Qgauss) then
           ! Unit for reading Gaussian hills [integer, default=0]
           call gtipar(com,'ugauss',ugauss,0)
           if (ugauss<=0 .or. ugauss>maxopen) call error ('shell_simul', 'Wrong unit for Gaussian hills in STRM order', faterr)
           if (unvec(ugauss)==-1) call error ('shell_simul', 'Wrong unit for Gaussian hills in STRM order', faterr)
           ugauss = unvec(ugauss)
           write(wrtline,*) 'Unit for Gaussian hills: ',ugauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Number of Gaussian functions [integer, default=0]
           call gtipar(com,'ngauss',ngauss,0)
           if (ngauss<=0) call error ('shell_simul', 'Wrong number of Gaussians functions in STRM order', faterr)
           write(wrtline,*) 'Number of Gaussian functions: ',ngauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Number of lines to skip in the hills file [integer, default=0]
           call gtipar(com,'ngskip',ngskip,0)
           if (ngskip<0) call error ('shell_simul', 'Wrong number of lines to skip in the hills file in STRM order', faterr)
           write(wrtline,*) 'Number of lines to skip in the hills file: ',ngskip
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Constant factor to pass from bias potential to FES [real, default=1.0]
           call gtdpar(com,'ctgauss',ctgauss,1.0)
           write(wrtline,*) 'Constant factor to pass from bias potential to FES: ',ctgauss
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Initial Gaussians for average FES [integer, default=ngauss]
           call gtipar(com,'nigauss',nigauss,ngauss)
           if (nigauss<=0 .or. nigauss>ngauss) call error ('shell_simul', 'Wrong number of initial Gaussians functions in STRM order', faterr)
           if (nigauss<ngauss) then
             write(wrtline,*) 'Initial Gaussians for average FES: ',nigauss
             write(outu,'(6x,a)') trim(adjustl(wrtline))
             ctgauss = ctgauss/(ngauss-nigauss) ! ctgauss includes number of states for computing mean FES 
           endif
           if (allocated(zgauss)) deallocate(zgauss)
           if (allocated(dgauss)) deallocate(dgauss)
           if (allocated(wgauss)) deallocate(wgauss)
           allocate(zgauss(ncv,ngauss),dgauss(ncv),wgauss(ngauss))
           call readgauss
         endif
         if (QBspline .or. Qbicubic) then
           ! Unit for reading CV grid points [integer, default=0]
           call gtipar(com,'ugrid',ugrid,0)
           if (ugrid<=0 .or. ugrid>maxopen) call error ('shell_simul', 'Wrong unit for CV grid points in STRM order', faterr)
           if (unvec(ugrid)==-1) call error ('shell_simul', 'Wrong unit for CV grid points in STRM order', faterr)
           ugrid = unvec(ugrid)
           write(wrtline,*) 'Unit for FES: ',ugrid
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           if (allocated(ng)) deallocate(ng)
           allocate(ng(ncv))
           ! Number of grid points in first CV direction [integer, default=0]
           call gtipar(com,'ng1',ng(1),0)
           if (ng(1)<=0) call error ('shell_simul', 'Wrong number of grid points in first CV direction in STRM order', faterr)
           mxgrid = ng(1)
           write(wrtline,*) 'Number of grid points in first CV direction: ',ng(1)
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           ! Number of grid points in second CV direction [integer, default=0]
           call gtipar(com,'ng2',ng(2),0)
           if (ng(2)<=0) call error ('shell_simul', 'Wrong number of grid points in second CV direction in STRM order', faterr)
           if (ng(2)>mxgrid) mxgrid = ng(2)
           write(wrtline,*) 'Number of grid points in second CV direction: ',ng(2)
           write(outu,'(6x,a)') trim(adjustl(wrtline))
           if (ncv==3) then
             ! Number of grid points in third CV direction [integer, default=0]
             call gtipar(com,'ng3',ng(3),0)
             if (ng(3)<=0) call error ('shell_simul', 'Wrong number of grid points in third CV direction in STRM order', faterr) 
             if (ng(3)>mxgrid) mxgrid = ng(3)         
             write(wrtline,*) 'Number of grid points in third CV direction: ',ng(3)
             write(outu,'(6x,a)') trim(adjustl(wrtline))
           endif
           if (allocated(dcel)) deallocate(dcel)
           if (allocated(idcel)) deallocate(idcel)
           if (allocated(xg)) deallocate(xg)
           if (allocated(fesg)) deallocate(fesg)
           allocate(dcel(ncv),idcel(ncv),xg(mxgrid,ncv))
           if (ncv==2) then
             allocate(fesg(ng(1)*ng(2)))
           else ! ncv = 3
             allocate(fesg(ng(1)*ng(2)*ng(3)))
           endif
           call readgrid
           if (Qbicubic) then 
             if (allocated(cg)) deallocate(cg)
             allocate(cg(4,4,(ng(1)-1)*(ng(2)-1)))
             call bsplinec
           endif
         endif
       endif
       ! Unit for free energy along a given path [integer, default=0]
       call gtipar(com,'ufe',ufe,0)
       if (ufe<=0 .or. ufe>maxopen) call error ('shell_simul', 'Wrong unit for free energy along a given path in STRM order', faterr)
       if (unvec(ufe)==-1) call error ('shell_simul', 'Wrong unit for free energy along a given path in STRM order', faterr)
       ufe = unvec(ufe)
       write(wrtline,*) 'Unit for free energy along a given path: ',ufe
       write(outu,'(6x,a)') trim(adjustl(wrtline))
       call stmfe
     endif
     write(outu,*)

  ! **********************************************************************
  elseif (wrd5.eq.'local') then
  !       ---------------
     write(wrtline,*) 'LOCAL FREE ENERGY ESTIMATOR'
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     write(wrtline,*) '---------------------------'
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Number of collective variables (CVs) [integer, default=0]
     call gtipar(com,'ncv',ncv,0)
     if (ncv<=0 .or. ncv>3) call error ('shell_simul', 'Wrong number of CVs in LOCAL order', faterr)
     write(wrtline,*) 'Number of CVs: ',ncv
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Unit for reading Gaussian hills [integer, default=0]
     call gtipar(com,'ugauss',ugauss,0)
     if (ugauss<=0 .or. ugauss>maxopen) call error ('shell_simul', 'Wrong unit for Gaussian hills in LOCAL order', faterr)
     if (unvec(ugauss)==-1) call error ('shell_simul', 'Wrong unit for Gaussian hills in LOCAL order', faterr)
     ugauss = unvec(ugauss)
     write(wrtline,*) 'Unit for Gaussian hills: ',ugauss
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Number of Gaussian functions [integer, default=0]
     call gtipar(com,'ngauss',ngauss,0)
     if (ngauss<=0) call error ('shell_simul', 'Wrong number of Gaussians functions in LOCAL order', faterr)
     write(wrtline,*) 'Number of Gaussian functions: ',ngauss
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Number of lines to skip in the hills file [integer, default=0]
     call gtipar(com,'ngskip',ngskip,0)
     if (ngskip<0) call error ('shell_simul', 'Wrong number of lines to skip in the hills file in LOCAL order', faterr)
     write(wrtline,*) 'Number of lines to skip in the hills file: ',ngskip
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Constant factor to pass from bias potential to FES [real, default=1.0]
     call gtdpar(com,'ctgauss',ctgauss,1.0)
     write(wrtline,*) 'Constant factor to pass from bias potential to FES: ',ctgauss
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     if (allocated(zgauss)) deallocate(zgauss)
     if (allocated(dgauss)) deallocate(dgauss)
     if (allocated(wgauss)) deallocate(wgauss)
     allocate(zgauss(ncv,ngauss),dgauss(ncv),wgauss(ngauss))
     call readgauss
     ! Temperature [real, default=300K]
     call gtdpar(com,'temp',temp,300.0)
     if (temp<=0.0) call error ('shell_simul', 'Wrong temperature in LOCAL order', faterr)
     kbt = kboltz*temp*avogadro*0.001
     write(wrtline,*) 'Thermal energy [KJ/mole]: ',kbt
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ikbt = 1.0/kbt
     ! Maximum system limit [real, default=0.0], minimum system limit [real, default=maximum value]
     if (allocated(cvlim)) deallocate(cvlim)
     allocate(cvlim(2,ncv))
     call gtdpar(com,'max1',cvlim(2,1),0.0)
     call gtdpar(com,'min1',cvlim(1,1),cvlim(2,1))
     if (cvlim(1,1)>=cvlim(2,1)) call error ('shell_simul1', 'Wrong system limits in LOCAL order', faterr)
     if (ncv>1) then
       call gtdpar(com,'max2',cvlim(2,2),0.0)
       call gtdpar(com,'min2',cvlim(1,2),cvlim(2,2))
       if (cvlim(1,2)>=cvlim(2,2)) call error ('shell_simul1', 'Wrong system limits in LOCAL order', faterr)  
     endif  
     if (ncv==3) then
       call gtdpar(com,'max3',cvlim(2,3),0.0)
       call gtdpar(com,'min3',cvlim(1,3),cvlim(2,3))
       if (cvlim(1,3)>=cvlim(2,3)) call error ('shell_simul1', 'Wrong system limits in LOCAL order', faterr)
     endif
     if (ncv==1) then
       write(wrtline,*) 'System dimension: [',cvlim(1,1),',',cvlim(2,1),']'
     else if (ncv==2) then
       write(wrtline,*) 'System dimension: [',cvlim(1,1),',',cvlim(2,1),'];[',cvlim(1,2),',',cvlim(2,2),']'
     else
       write(wrtline,*) 'System dimension: [',cvlim(1,1),',',cvlim(2,1),'];[',cvlim(1,2),',',cvlim(2,2),'];[',cvlim(1,3),',',cvlim(2,3),']'
     endif
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Number of grid points in CV space [integer, default=0]
     if (allocated(ngrid)) deallocate(ngrid)
     allocate(ngrid(ncv))
     call gtipar(com,'grid1',ngrid(1),0)
     if (ngrid(1)<=0) call error ('shell_simul', 'Wrong number of grid points in LOCAL order', faterr)
     if (ncv>1) then
       call gtipar(com,'grid2',ngrid(2),0)
       if (ngrid(2)<=0) call error ('shell_simul', 'Wrong number of grid points in LOCAL order', faterr)
     endif
     if (ncv==3) then
       call gtipar(com,'grid3',ngrid(3),0)
       if (ngrid(3)<=0) call error ('shell_simul', 'Wrong number of grid points in LOCAL order', faterr)
     endif
     if (ncv==1) then
       write(wrtline,*) 'Number of grid points: ',ngrid(1)
     else if (ncv==2) then
       write(wrtline,*) 'Number of grid points: ',ngrid(1)*ngrid(2)
     else
       write(wrtline,*) 'Number of grid points: ',ngrid(1)*ngrid(2)*ngrid(3)
     endif 
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! CV point [real, default=0.0]
     if (allocated(cvpoint)) deallocate(cvpoint)
     allocate(cvpoint(ncv))
     call gtdpar (com,'cv1',cvpoint(1),0.0)
     if (cvpoint(1)<cvlim(1,1) .or. cvpoint(1)>cvlim(2,1)) call error ('shell_simul', 'Wrong CV value in LOCAL order', faterr)
     if (ncv>1) then
       call gtdpar (com,'cv2',cvpoint(2),0.0)
       if (cvpoint(2)<cvlim(1,2) .or. cvpoint(2)>cvlim(2,2)) call error ('shell_simul', 'Wrong CV value in LOCAL order', faterr)
     endif
     if (ncv==3) then
       call gtdpar (com,'cv3',cvpoint(3),0.0)
       if (cvpoint(3)<cvlim(1,3) .or. cvpoint(3)>cvlim(2,3)) call error ('shell_simul', 'Wrong CV value in LOCAL order', faterr)
     endif
     if (ncv==1) then 
       write(wrtline,*) 'CV: ',cvpoint(1)
     else if (ncv==2) then
       write(wrtline,*) 'CV: (',cvpoint(1),',',cvpoint(2),')'
     else
       write(wrtline,*) 'CV: (',cvpoint(1),',',cvpoint(2),',',cvpoint(3),')'
     endif
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Unit for FES estimator [integer, default=0]
     call gtipar(com,'ufes',ufel,0)
     if (ufel<=0 .or. ufel>maxopen) call error ('shell_simul', 'Wrong unit for FES estimator in STRM order', faterr)
     if (unvec(ufel)==-1) call error ('shell_simul', 'Wrong unit for FES estimator in STRM order', faterr)
     ufel = unvec(ufel)
     write(wrtline,*) 'Unit for FES estimator: ',ufel
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     if (ncv==1) then
       call FESlocal1
     else if (ncv==2) then
       call FESlocal2
     else
       call FESlocal3
     endif
     write(outu,*)

 ! **************************************************************************
  elseif (wrd5.eq.'fes') then
 !        ---------------
     ! Number of collective variables (CVs) [integer, default=0]
     call gtipar(com,'ncv',ncv,0)
     if (ncv<2 .or. ncv>3) call error ('shell_simul', 'Wrong number of CVs in FES order', faterr)
     write(wrtline,*) 'FREE ENERGY SURFACE'
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     write(wrtline,*) '-------------------'
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     QBspline = check(com,'Bspline')
     Qbicubic = check(com,'bicubic')
     if (QBspline .and. Qbicubic) call error ('shell_simul', 'B-spline and  bicubic interpolation can not be invoked simultaneously in FES order', faterr)
     if (Qbicubic .and. ncv/=2) call error ('shell_simul', 'ncv /= 2 is not valid for bicubic interpolation in FES order', faterr)
     if (.not.QBspline .and. .not.Qbicubic) call error ('shell_simul', 'B-spline and bicubic interpolation are not invoked in FES order', faterr)
     if (QBspline) then
       write(outu,'(6x,a)') 'B-spline interpolation for estimating the FES'
     else ! Qbicubic
       write(outu,'(6x,a)') 'Bicubic interpolation for estimating the FES'
     endif
     ! Unit for reading CV grid points [integer, default=0]
     call gtipar(com,'ugrid',ugrid,0)
     if (ugrid<=0 .or. ugrid>maxopen) call error ('shell_simul', 'Wrong unit for CV grid points in FES order', faterr)
     if (unvec(ugrid)==-1) call error ('shell_simul', 'Wrong unit for CV grid points in FES order', faterr)
     ugrid = unvec(ugrid)
     write(wrtline,*) 'Unit for FES: ',ugrid
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     if (allocated(ng)) deallocate(ng)
     allocate(ng(ncv))
     ! Number of grid points in first CV direction [integer, default=0]
     call gtipar(com,'ng1',ng(1),0)
     if (ng(1)<=0) call error ('shell_simul', 'Wrong number of grid points in first CV direction in FES order', faterr)
     mxgrid = ng(1)
     write(wrtline,*) 'Number of grid points in first CV direction: ',ng(1)
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! Number of grid points in second CV direction [integer, default=0]
     call gtipar(com,'ng2',ng(2),0)
     if (ng(2)<=0) call error ('shell_simul', 'Wrong number of grid points in second CV direction in FES order', faterr)
     if (ng(2)>mxgrid) mxgrid = ng(2)
         write(wrtline,*) 'Number of grid points in second CV direction: ',ng(2)
         write(outu,'(6x,a)') trim(adjustl(wrtline))
     if (ncv==3) then
       ! Number of grid points in third CV direction [integer, default=0]
       call gtipar(com,'ng3',ng(3),0)
       if (ng(3)<=0) call error ('shell_simul', 'Wrong number of grid points in third CV direction in FES order', faterr)
       if (ng(3)>mxgrid) mxgrid = ng(3)
       write(wrtline,*) 'Number of grid points in third CV direction: ',ng(3)
       write(outu,'(6x,a)') trim(adjustl(wrtline))
     endif
     if (allocated(dcel)) deallocate(dcel)
     if (allocated(idcel)) deallocate(idcel)
     if (allocated(xg)) deallocate(xg)
     if (allocated(fesg)) deallocate(fesg)
     allocate(dcel(ncv),idcel(ncv),xg(mxgrid,ncv))
     if (ncv==2) then
       allocate(fesg(ng(1)*ng(2)))
     else ! ncv = 3
       allocate(fesg(ng(1)*ng(2)*ng(3)))
     endif
     call readgrid
     if (Qbicubic) then
       if (allocated(cg)) deallocate(cg)
       allocate(cg(4,4,(ng(1)-1)*(ng(2)-1)))
       call bsplinec
     endif
     ! Unit for reading CV grid points [integer, default=0]
     call gtipar(com,'ugridw',ugrid2,0)
     if (ugrid2<=0 .or. ugrid2>maxopen) call error ('shell_simul', 'Wrong unit for CV grid points in FES order', faterr)
     if (unvec(ugrid2)==-1) call error ('shell_simul', 'Wrong unit for CV grid points in FES order', faterr)
     ugrid2 = unvec(ugrid2)
     write(wrtline,*) 'Unit for new FES: ',ugrid2
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     if (allocated(ng2)) deallocate(ng2)
     allocate(ng2(ncv))
     ! New number of grid points in first CV direction [integer, default=0]
     call gtipar(com,'ng1w',ng2(1),0)
     if (ng2(1)<=0) call error ('shell_simul', 'Wrong new number of grid points in first CV direction in FES order', faterr)
     write(wrtline,*) 'New number of grid points in first CV direction: ',ng2(1)
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     ! New number of grid points in second CV direction [integer, default=0]
     call gtipar(com,'ng2w',ng2(2),0)
     if (ng2(2)<=0) call error ('shell_simul', 'Wrong new number of grid points in second CV direction in FES order', faterr)
     write(wrtline,*) 'New number of grid points in second CV direction: ',ng2(2)
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     if (ncv==3) then
       ! New number of grid points in third CV direction [integer, default=0]
       call gtipar(com,'ng3w',ng2(3),0)
       if (ng2(3)<=0) call error ('shell_simul', 'Wrong new number of grid points in third CV direction in FES order', faterr)
       write(wrtline,*) 'New number of grid points in third CV direction: ',ng2(3)
       write(outu,'(6x,a)') trim(adjustl(wrtline))
     endif
     call writegrid
     write(outu,*)

  ! **************************************************************************
  elseif (wrd5.eq.'exit') then
  !        ---------------
     finish = timer()-start
     write(wrtline,*) 'cpu time:  ',finish,' msec'
     write(outu,'(6x,a)') trim(adjustl(wrtline))
     call DATE_AND_TIME(date, time, zone, values)
     write(outu,'(6x,a,1x,i2,a,i2,a,i2)') 'finished at (HH:MM.SS):',values(5),':',values(6),'.',values(7)
     write(outu,*)
     logfinal = .true.
  else
    write(outu,'(6x,a)') '*ERROR*  Unrecognized command:'//wrd5
    write(outu,*)
  endif
enddo

return
end
