SUBROUTINE spline(yg,n,h,y2g)
! yg(1),...,yg(n)   => tabulated function at the grid points
! h                 => grid size
! y2g(1),...,y2g(n) => second derivatives of the interpolating function at the grid points

implicit none
! INPUT VARIABLES
integer n
real h, yg(n) 
! OUTPUT VARIABLE
real y2g(n)
! LOCAL VARIABLES
integer i
real ih2, p, u(n)

ih2 = 1.0/h**2
y2g(1) = 0.0 ! The lower boundary condition is set to be 'natural'
! Descomposition loop of the tridiagonal algorithm
u(1) = 0.0
do i = 2, n-1
  p = 1.0/(0.5*y2g(i-1)+2.0)
  y2g(i) = -0.5*p
  u(i) = (3.0*ih2*(yg(i+1)-2.0*yg(i)+yg(i-1))-0.5*u(i-1))*p
enddo
y2g(n) = 0.0 ! The upper boundary condition is set to be 'natural'
! Backsubstitution loop of the tridiagonal algorithm
do i = n-1, 1, -1
  y2g(i) = y2g(i)*y2g(i+1) + u(i)
enddo

END SUBROUTINE spline
