SUBROUTINE stm
! String algorithm
! Note: mig > 3 (this is not tested in this routine)
use ftsmod
use errormod

implicit none
! local variables
integer iter, i, j, aval, afunc
real zinit(ncv,mig), imig
real vec(ncv), force(ncv), tmp(ncv,mig), tmp2(ncv,mig), dist, distmx
real  zold(ncv,mig)
real  deucl(mig), cte, idt 
character :: wrtline*2048
logical*1 convergence

if (Qestring) then
  do i = 1, mig
    do j = 1, ncv
      zinit(j,i) = zrep(j,i)
    enddo
  enddo
  imig = 1.0/mig
endif
if (Qconv) idt = 1.0/dt
iter = 1
convergence = .false.
do while (.not.convergence .and. iter<=ftsit)
  if (Qconv) then
    do i = 1, mig
      do j = 1, ncv
        zold(j,i) = zrep(j,i)
      enddo
    enddo
  endif
  ! ** Step1: evolve the images using a first-order Euler algorithm
  do i = 1, mig
    call gradpmf(vec,i) ! force vector associated to the FES
    force = matmul(Mm,vec)
    do j = 1, ncv
       tmp(j,i) = zrep(j,i) + dt*force(j) 
    enddo
    if (Qangcv) then
      do j = 1, ncv
        if (angcv(j) .and. abs(tmp(j,i)).gt.dcv(j)) tmp(j,i) = sign(dcv(j),tmp(j,i))
      enddo
    endif
  enddo ! next image
  ! ** Step 2: smooth the string using a convex interpolation
  do j = 1, ncv
    tmp2(j,1) = tmp(j,1)
    tmp2(j,mig) = tmp(j,mig)
  enddo
  do i = 2, mig-1
    do j = 1, ncv
      tmp2(j,i) = (1.0-kappt)*tmp(j,i) + 0.5*kappt*(tmp(j,i-1)+tmp(j,i+1))
    enddo
  enddo
  ! ** Step 3: reparametrization of the string
  ! **** 3.b.i) euclidean distances between consecutives images
  do i = 2, mig 
    deucl(i) = 0.0 
    do j = 1, ncv 
      deucl(i) = deucl(i) + (tmp2(j,i)-tmp2(j,i-1))**2 
    enddo 
    deucl(i) = sqrt(deucl(i)) 
  enddo 
  ! **** 3.b.ii) length of the piecewise linear curve up to a given image (recurrence relation)
  Lcur(1) = 0.0 
  do i = 2, mig  
    Lcur(i) = Lcur(i-1) + deucl(i) 
    deucl(i) = 1.0/deucl(i) 
  enddo 
  ! **** 3.b.iii) distance from image 1 up to the point at which we need to set a new given image (recurrence relation)
  cte = Lcur(mig)/(mig-1.0) 
  ldist(1) = 0.0 
  do i = 2, mig 
    ldist(i) = ldist(i-1) + cte 
  enddo 
  ! **** 3.b.iv) new equidistant images
  do j = 1, ncv
    zrep(j,1) = tmp2(j,1)
    zrep(j,mig) = tmp2(j,mig) 
  enddo
  do i = 2, mig-1 
    aval = afunc(i) 
    do j = 1, ncv 
      zrep(j,i) = tmp2(j,aval-1) + (ldist(i)-Lcur(aval-1))*(tmp2(j,aval)-tmp2(j,aval-1))*deucl(aval) 
    enddo 
  enddo 
  ! ** Euclidean distance of the string from the initial condition
  if (Qestring) then
    dist = 0.0
    do i = 1, mig
      do j = 1, ncv
        dist = dist + (zrep(j,i)-zinit(j,i))**2
      enddo
    end do
    dist = sqrt(dist)*imig
    write(wrtline,*) iter,dist
    write(ustring,'(a)') trim(adjustl(wrtline))
  endif
  ! ** Check convergence
  if (Qconv) then
    ! **** maximum displacement of the string
    distmx = 0.0
    do i = 1, mig
      dist = 0.0
      do j = 1, ncv
        dist = dist + (zrep(j,i)-zold(j,i))**2
      enddo
      dist = sqrt(dist)*idt
      if (distmx<dist) distmx = dist
    enddo
    convergence = distmx<ftstol
  endif
  iter = iter + 1
enddo
if (Qconv .and. .not.convergence) call error ('stm', 'Convergence is not achieved', faterr)

! Print images file
if (Qfts) then
  write(wrtline,*) '! Angle CVs?:'
  write(ufts,'(a)') trim(adjustl(wrtline))
  write(wrtline,*) Qangcv
  write(ufts,'(a)') trim(adjustl(wrtline))
  if (Qangcv) then
    write(wrtline,*) '! Constant module for angle CVs:'
    write(ufts,'(a)') trim(adjustl(wrtline))
    write(wrtline,*) (dcv(j),j=1,ncv)
    write(ufts,'(a)') trim(adjustl(wrtline))
  endif
  write(wrtline,*) '! Images (ncv dimension):' 
  write(ufts,'(a)') trim(adjustl(wrtline))
  do i = 1, mig
    write(wrtline,*) (zrep(j,i),j=1,ncv)
    write(ufts,'(a)') trim(adjustl(wrtline))
  enddo
endif

return
END SUBROUTINE stm
